/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.core;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Arrays;
import java.util.Stack;


public class BlockConstraint {
	private int numSize ; 
	// ȉ̂ƂO
	// [0 ... n) ͏c
	// [n ... n+n) ͉
	// ȍ~̓ubN
	private final ArrayList<Integer[]> block;
	
	// ZĂ][@[cell_idx][...]
	private final int[][] blockWhereCellBelong ;
	
	// Q̃ubNɊ܂܂Z̔z@@[block_idx1][block_idx2]
	private final Integer[][][] blockSetIntersection ;

	private ArrayList<Pair<Integer, Integer> > intesectionList;

	ArrayList<Pair<Integer, Integer> > getIntersetionList(){
		return intesectionList;
	}
	
	public BlockConstraint(ArrayList<Integer[]> block , int n){
		this.block = block;
		this.numSize = n ;
		//setupBlock
		n = n*n;
		blockWhereCellBelong = new int[n][];
		int[] hist = new int[n]; 
		// ez񒷂炩ߋ߂Ă
		for(Integer[] seq : block) for(Integer val : seq){
				hist[val]++;
			}
		for(int i=0;i<n;i++){
			blockWhereCellBelong[i] = new int[hist[i]]; 
		}
		int[] cursor = new int[n];
		for(int i=0;i<block.size();i++)
		{
			for(int j=0;j<block.get(i).length;j++)
			{
				int x = block.get(i)[j];
				blockWhereCellBelong[x][cursor[x]++] = i ;
			}
		}

		// blockSetIntesection initialize
		for(int i=0;i<block.size();i++) Arrays.sort(block.get(i));
		blockSetIntersection = new Integer[block.size()][block.size()][];
		intesectionList = new ArrayList<Pair<Integer,Integer> >();
		for(int i=0;i<block.size();i++)
			for(int j=i+1;j<block.size();j++)
			{
				LinkedList<Integer> list = new LinkedList<Integer>();
				for(Integer x : block.get(i)){
					if(Arrays.binarySearch(block.get(j), x) >= 0)
						list.add(x);
				}
				blockSetIntersection[i][j] = (Integer[])list.toArray(new Integer[0]);
				if(blockSetIntersection[i][j].length >= 2){
					Pair<Integer,Integer> pair = new Pair<Integer,Integer>(i,j);
					intesectionList.add(pair);
				}
			}
	}
	public int getBlockSize() { return block.size(); }
	public ArrayList<Integer[]> getBlock(){
		return block;
	}
	public int[][] getBlockWhereCellBelong(){
		return blockWhereCellBelong;
	}
	public int[] getBlockWhereCellBelong(int cell_idx){
		return blockWhereCellBelong[cell_idx];
	}
	public Integer[][][] getBlockSetIntesection(){
		return blockSetIntersection;
	}
	public Integer[] getBlockArray(){
		Integer[] array  = new Integer[numSize*numSize] ;
		for(int i=numSize+numSize,lbl=1;i<block.size();i++,lbl++){
			for(int cell_idx : block.get(i)){
				array[cell_idx] = lbl ; 
			}
		}
		return array;
	}
	/** 
	 * f[^AubNf[^ɕϊ
	 * @param numSize TCY
	 * @param vLine c
	 * @param hLine@
	 * @return
	 */
	public static int[] getBlockArrayFromLine(int numSize , boolean[][] vLine , boolean[][] hLine) {
		int[] array  = new int[numSize*numSize] ;
		Arrays.fill(array, -1);
		final int dx[] = {1,0,-1,0};
		final int dy[] = {0,-1,0,1};
		int color = 0;
		for(int i=0;i<array.length;i++) if(array[i]==-1){
			Stack<Integer> st = new Stack<Integer>();
			st.push(i); color++;
			int num = 0;
			while(!st.empty()){
				int now = st.pop();
				if(array[now]!=-1) continue;
				array[now] = color;
				num++;
				int row = now/numSize;
				int col = now%numSize;
				for(int j=0;j<4;j++){
					int nrow = row + dy[j];
					int ncol = col + dx[j];
					int nidx = nrow*numSize+ncol;
					if(0<=nrow && nrow < numSize && 0 <= ncol && ncol < numSize && array[nidx]==-1){
						if(dy[j]==0){
							if(vLine[row][Math.min(col, ncol)]) continue;
						}else{
							if(hLine[Math.min(row, nrow)][col]) continue;							
						}
						st.push(nidx);
					}
				}
			}
			if(num != numSize) return null;
		}
		return array;
	}
	//ArrayList<Pair<>>
}
