/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.core;

import java.io.File;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import jp.gr.puzzle.npv2.xml.*;


import org.w3c.dom.Document;

public class Generator {
	public final int RESERVED = -1 , SPACE = 0;
	public final SolverMethod seedMethod ;
	
	public SolverMethod method ; 
	// }Xɓ鐔̌
	private int numSize ;
	
	// Iȃqg
	private int[] hint;
	
	// SEED
	private int[] seed;
	
	// ̏iÎIj
	private int[] hidden;

	public int forbidden = -1;
	
	LinkedList<Integer> hiddenList;
	
	LinkedList<Integer> hintList;
	
	BlockConstraint block;
	
	int[][] group;

	Status original ;
	Status statussub ;
	Status group_status ;	
	
	private boolean interruption;
	public void setInterruption( boolean b ) {
		this.interruption = b;
	}
	
	public void setForbidden(int x){
		this.forbidden = x ;
	}
	
	public void showDebugData(){
		System.out.println("num : " + numSize);
		System.out.print("hint : ");
		for(int x : hintList) System.out.print(" " + x); System.out.println();
		System.out.print("hidden : ");
		for(int x : hiddenList) System.out.print(" " + x); System.out.println();
	}
	public Generator(int num_size , int[] hint , int[] hidden ,
			BlockConstraint block ) {
		this.numSize = num_size;
		this.hint = hint;
		this.hidden  = hidden;
		this.block = block;
		this.seedMethod = new SolverMethod();
		this.method = new SolverMethod();
		original = new Status(numSize , block);
		statussub = new Status(numSize , block);
		group_status = new Status(numSize , block);
		seedMethod.localization = true;
		seedMethod.nakedPair = true;
		seedMethod.hiddenPair = true;
		initialize();
	}
	public int[] getSeed(){
		return seed;
	}
	public void setMethod(SolverMethod method){
		this.method = method;
	}
	public Generator(String XMLfilename){
		this.loadXML(XMLfilename);
		this.seedMethod = new SolverMethod();
		seedMethod.localization = true;
		seedMethod.nakedPair = true;
		seedMethod.hiddenPair = true;
		original = new Status(numSize , block);
		statussub = new Status(numSize , block);
		group_status = new Status(numSize , block);
		initialize();
	}
	private void initialize(){
		
		hiddenList = new LinkedList<Integer>();
		for(int i=0;i<hidden.length;i++) 
			if(hidden[i] != 0)	hiddenList.add(i); 

		hintList = new LinkedList<Integer>();
		for(int i=0;i<hint.length;i++)
			if(hint[i]!=0) hintList.add(i);
		
		{  
			int GROUP = Utility.sqrt(hintList.size()) ;
			if(GROUP == 0) GROUP = 1;
			group = new int [GROUP][];
			int remain = hintList.size() % GROUP;
			int unit   = hintList.size() / GROUP;
			Iterator<Integer> it = hintList.iterator();
			//int cnt = 0;
			for(int i=0;i<GROUP;i++){
				int elements = unit ; 
				if(i < remain) elements++;
				group[i] = new int[elements];
				for(int j=0;j<elements;j++){
					//System.out.println(cnt++ + " " + i + " " + j + " " + hintList.size());
					group[i][j] = it.next();
				}
			}
		}
	}

	private int[] generateSeedSub(){
		int[] stat = new int[hidden.length];
		System.arraycopy(hidden, 0, stat, 0, hidden.length);
		Status status = new Status(numSize, block); 
		{
			/*Solver solver = new Solver(new Status(stat , numSize, block));
			solver.setMethod(seedMethod);*/
			for(int i=0;i<stat.length;i++) {
				if(stat[i]>0) {
					Solver.addNumber(status,i, stat[i]);
				}
			}
			status = Solver.answer(status,seedMethod);
			if(status.getKindOfAnswer() == Solver.KindOfAnswer.IRREGULAR_PROBLEM){
				System.out.println("NO");
			}
		}
		for(int i=0;i<stat.length;i++){
			if(status.isEmptyCell(i) == false) continue;
			int candnum = status.getCandCountOfCell(i);
			if(candnum==0) {
				//System.out.println("NO ANSWER");
				return null;
			}
			if(status.isNoAnswer()){
				//System.out.println("NO ANSWER 2"); TODO
				return null;
			}
			int r = Utility.random(candnum) ;
			int x = status.getNthCandOfCell(i, r );
			if(hint[i]!=0 && x == forbidden){
				if(candnum==1) return null;
				x = status.getNthCandOfCell(i,(r+1)%candnum);
			}
			//status.assignValue(i, x); 
			//Solver solver = new Solver(status);
			//solver.setMethod(seedMethod);
			Solver.addNumber(status, i, x);
			status = Solver.answer(status , seedMethod);
		}
		if(forbidden > 0)
			for(int cell_idx : hintList)
				if(status.getCell(cell_idx)==forbidden) return null;
		if(status.getSpaceCount()!=0) return null;
		return status.getCell();
	}
	private int [] generateSeed(){
		int[] val ; 
		int failed_num = 0; 
		while(true){
			if( interruption )
				return null;
			val = generateSeedSub() ; 
			if(val != null) {
				//System.out.println("seed : " + failed_num);
				return val;
			}
			failed_num ++ ;
			if(failed_num > 100) {
				//	System.out.println("Not seed gained.");
				return null;
			}
		}
	}
	public boolean fitToHidden(int [] cell){
		for(int i : hiddenList) {
			if(cell[i] != hidden[i]) return false;
		}
		return true;
	}
	public int[] generate(){
		//if(true) return generateold();
		interruption = false;
		seed = generateSeed(); 
		if(seed == null) return null;
		int[] stat = new int[hidden.length];
		for(int cell_idx : hintList){
			stat[cell_idx] = seed[cell_idx];
		}
		//Solver solver = new Solver(new Status(stat , numSize , block));
		Status status = new Status(numSize , block);
		status.setUniqueMethod(this.method.unique);
		for(int i=0;i<stat.length;i++)
			if(stat[i]>0){
				Solver.addNumber(status , i, stat[i]);
			}
		status = Solver.answer(status, this.method);
		if(status.getKindOfAnswer() == Solver.KindOfAnswer.UNIQUE_ANSWER){
			return stat;
		}	
		original    .clear();
		statussub   .clear();
		group_status.clear();
		original    .setUniqueMethod(this.method.unique);
		statussub   .setUniqueMethod(this.method.unique);
		group_status.setUniqueMethod(this.method.unique);
		Collections.shuffle(hintList);
		int zero = status.getSpaceCount();
		if(zero == 0) return stat;
		boolean yet = true; 
		while(yet){
			yet = false;
			for(int i = 0 ; i < group.length ; i++){
				int[] g= group[i];
				group_status.clear();
				for(int j=0;j<group.length;j++) if(i!=j){
					for(int x : group[j]){
						Solver.addNumber(group_status, x , stat[x]);
					}
				}
				for(int cell_idx : g){
					if(hidden[cell_idx]!=0) continue;
					if( interruption )
						return null;					
					int pre = stat[cell_idx] ;
					
					stat[cell_idx] = 0 ;
					original.copyStatusToThis(group_status);
					//original = group_status.dup();
					for(int x : g) if(x != cell_idx) {
						Solver.addNumber(original, x, stat[x]);
					}
					if(original.getCandCountOfCell(cell_idx)<=1){
						stat[cell_idx] = pre ; 
						continue;
					}
					LinkedList<Integer> list = original.getCandidateList(cell_idx);
					
					if(list.contains(pre)==false){
						list.addLast(pre);
					}
					stat[cell_idx] = pre ; 
					
					while(list.size() != 0 && list.getFirst() != pre) {
						list.addLast(list.removeFirst());
					}
					list.removeFirst();
					
					for(int next_cond : list){
						if(next_cond==pre) continue;
						if(next_cond==forbidden) continue;
						statussub.copyStatusToThis(original);
						//Solver solve2 = new Solver(statussub);
						//solve2.addNumber(cell_idx, next_cond);
						Solver.addNumber(statussub, cell_idx, next_cond);
						statussub = Solver.answer(statussub , method);
						int space = statussub.getSpaceCount();
						//int candnum  = statussub.getCandCountOfCell(cell_idx);
						if(statussub.isNoAnswer()){		
							//System.out.println("no answer");
						}else if(fitToHidden(statussub.getCell())==false){
							//System.out.println("paradox");		
						}else {
							if(zero > space){			
								zero = space;
								stat[cell_idx] = next_cond;			
								if(zero == 0) {
									return stat;
								}
								yet = true; break;
							}
						}
					}
					if(yet) break;
					
				}
			}
		}
		if(zero != 0) return null;
		return stat;
	}
	
 
	private void loadXML(String XMLfilename){
		try{
		    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
		    DocumentBuilder builder = dbfactory.newDocumentBuilder();
		    Document doc = builder.parse(new File(XMLfilename));
		    Problem problem = new Problem(doc);
		    Constraint constraint = problem.getConstraint();
		    this.numSize = Integer.parseInt(problem.getSize());
		    this.hint     = Utility.toIntArray(problem.getHint()   , numSize * numSize);
		    this.hidden   = Utility.toIntArray(problem.getHidden() , numSize * numSize);
		    this.seed     = Utility.toIntArray(problem.getSeed()   , numSize * numSize);
		    this.block    = new BlockConstraint(Utility.toBlockArrayFromConstraint(constraint, numSize) , numSize);
		    if(this.block == null){
		    	System.out.println("ERR");
		    }
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	public static void main(String[] args) {
		Generator gen = new Generator("gen.xml");
		System.out.println("MAIN");

		for(int loop=1;loop>0;loop--){
		
		long starttime = System.currentTimeMillis();
		
		int[] problem = null;
		int time = 1;
		while(problem == null){
			System.out.println("GENERATE : " + time++);
			problem = gen.generate();
		}
		long endtime = System.currentTimeMillis();
		System.out.println(" time : " + (endtime - starttime));
		System.out.println((double)(endtime - starttime) / time);
		int n = gen.numSize;
		for(int i=0;i<n;i++){
			for(int j=0;j<n;j++){
				System.out.print(" " + problem[i*n+j]);
			}
			System.out.println();
		}
		
		//Solver solve = new Solver(new Status(problem,gen.num_size,gen.block));
		//Status s = solve.solve();
		//s.showDebugData();
		
		}
	}
}
