/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.core;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import jp.gr.puzzle.npv2.xml.Constraint;
import jp.gr.puzzle.npv2.xml.Group;
import jp.gr.puzzle.npv2.xml.Problem;
import jp.gr.puzzle.npv2.xml.Question;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

public class NumberPlaceFile {
	private int numSize = -1; 
	private boolean[] hint = null;
	private int[] hidden = null, answer = null, problem = null ;
	private int[] blockarray = null;  
	private boolean is_diagonal = false;  
	private boolean default_block = true; 
	private int difficult = -1;
	public NumberPlaceFile(){}
	public NumberPlaceFile(File file) throws ParserConfigurationException , SAXException , IOException {
		Load(file);
	}
	public boolean isDefaultBlock(){
		return default_block;
	}
	public void setNumSize(int numSize){
		this.numSize = numSize;
	}
	public int getNumSize(){
		return this.numSize;
	}
	public void setHint(boolean[] hint){
		this.hint = hint;
	}
	public boolean [] getHint(){
		return this.hint ; 
	}
	public void setHidden(int[] hidden){
		this.hidden = hidden;
	}
	public int [] getHidden(){
		return this.hidden ; 
	}
	public void setAnswer(int[] answer){
		this.answer = answer;
	}
	public int [] getAnswer(){
		return this.answer ; 
	}
	public void setProblem(int[] problem){
		this.problem = problem;
	}
	public int [] getProblem(){
		return this.problem ; 
	}
	public void setBlockArray(int[] blockarray){
		this.blockarray = blockarray;
	}
	public int [] getBlockArray(){
		return this.blockarray ; 
	}
	public boolean isDiagonal(){
		return is_diagonal;
	}
	public void setIsDiagonal(boolean is_diagonal){
		this.is_diagonal = is_diagonal;
	}
	public void setDifficult(int diff){
		this.difficult = diff;
	}
	public int getDifficult(){
		return this.difficult;
	}
	public void Load(File file) throws ParserConfigurationException , SAXException , IOException {
	    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = dbfactory.newDocumentBuilder();
	    Document doc = builder.parse(file);
	    Problem problem = new Problem(doc);
	    Constraint constraint = problem.getConstraint();
	    this.numSize = Integer.parseInt(problem.getSize()); 
	    this.hint     = Utility.toBooleanArray(problem.getHint()    , numSize * numSize);
	    this.hidden   = Utility.toIntArray    (problem.getHidden()  , numSize * numSize);
	    this.answer   = Utility.toIntArray    (problem.getAnswer()  , numSize * numSize);
	    this.problem  = Utility.toIntArray    (problem.getQuestion().getContent(), numSize * numSize);
	    this.is_diagonal = 
	    		constraint.getDiagonal() != null &&
	    		constraint.getDiagonal().equals(Constraint.DIAGONAL_ON);
	    this.default_block	= (constraint.getDefaultBlock()!=null&&constraint.getDefaultBlock().equals("on"));
	    if(default_block){
	    	this.blockarray = null ; 
	    }else{
	    	for(Group group : constraint.getGroup())
			{
				this.blockarray = Utility.toIntArray(group.getContent(),numSize*numSize);
				break;
			}
	    }
	}
	public void Save(File file){
	    Problem problem = new Problem();
	    problem.setSize(Integer.toString(numSize));
	    problem.setName("Number Place");
	    problem.setAuthor("Number Place Generator");
	    if(this.answer != null) problem.setAnswer(Utility.toStringFromArray(this.answer));
	    Constraint constraint = new Constraint();
	    constraint.setDiagonal(isDiagonal()?Constraint.DIAGONAL_ON:Constraint.DIAGONAL_OFF);
	    Group group = new Group();
	    Question q = new Question();
	    /*if(blockArray==null)
	    {
			JOptionPane.showMessageDialog(
					this , "invalid block" , "Error : block status" ,
					JOptionPane.ERROR_MESSAGE
				);
			return ;
	    }*/
	    group.setContent(Utility.toStringFromArray(blockarray));
	    group.setBlock(Group.BLOCK_ON);
	    constraint.setGroup(group);
	    problem.setQuestion(q);
	    if(this.problem != null) problem.getQuestion().setContent(Utility.toStringFromArray(this.problem));
	    problem.getQuestion().setDifficult(Integer.toString(getDifficult()));
	    ///problem.setQuestion(Utility.toStringFromArray(this.problem));
	    problem.setAnswer(Utility.toStringFromArray(this.answer));
	    if(this.hint != null) problem.setHint(Utility.toStringFromArray(this.hint));
	    if(this.hidden != null) problem.setHidden(Utility.toStringFromArray(this.hidden));
	    problem.setConstraint(constraint);
	    PrintWriter pw = null;
	    try{
	    	pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));	    	
	    	pw.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
	    	problem.makeTextElement(pw);
	    }catch (Exception e){
	    	e.printStackTrace();
	    }finally{
	    	if(pw != null) pw.close();
	    }
	}
}
