/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.core;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import jp.gr.puzzle.npv2.xml.*;


import org.w3c.dom.Document;

public class ProblemBuilder
{
	// block 
	boolean vertical      = true;
	boolean horizontal     = true;
	boolean diagonal      = false;
	boolean default_block = true;
	int[] cell;
	int numSize ;
	ArrayList<Integer[]> group;
	public ProblemBuilder() {
		group = new ArrayList<Integer[]>();
	}
	/*
	private void printDebugData()
	{
		System.out.println("vertical " + vertical);
		System.out.println("horizontal " + horizontal);
		System.out.println("diagonal " + diagonal);
		for(int i=0;i<numSize;i++){
			for(int j=0;j<numSize;j++){
				System.out.print(" " + cell[i*numSize+j]);
			}
			System.out.println();
		}
	}
	*/
	/**
	 * c̃ubNʂZbg
	 *
	 */
	private void addBlockVertical(ProblemContent pc){
		Utility.addBlockVertical(pc.getBlock(), numSize);
	}	
	/**
	 * ̃ubNʂZbg
	 *
	 */
	private void addBlockHorizontal(ProblemContent pc){
		Utility.addBlockHorizontal(pc.getBlock(), numSize);
	}
	/**
	 * w * h ̃ubNt
	 * @param w@ubN̕
	 * @param h@ubN̍
	 * w * h = num_size łȂ΂ȂȂ
	 */
	public void addBlockRectangle(int w , int h , ProblemContent pc)
	{
		Utility.addBlockRectangle(w, h, pc.getBlock(), numSize);
	}	
	
	/**
	 * c̃ubNʂZbg
	 *
	 */
	public void addBlockVerticalAndHorizontal(ProblemContent pc){
		addBlockVertical(pc);
		addBlockHorizontal(pc);
	}
	/**
	 * Ίp
	 */
	public void addBlockDiagonal(ProblemContent pc)
	{
		Utility.addBlockDiagonal(pc.getBlock(), numSize);
	}

	public void addBlockByArray(Integer [] array , ProblemContent pc)
	{
		Utility.addBlockByArray(array, pc.getBlock(), numSize);
	}
	public void addGroup(Integer[] array){
		group.add(array);
	}
	public ProblemContent build(){
		ProblemContent problem = new ProblemContent(numSize , cell);
		if(vertical ) addBlockVertical (problem);
		if(horizontal) addBlockHorizontal(problem);
		if(diagonal)  addBlockDiagonal (problem);
		int sq = Utility.sqrt(numSize);
		if(default_block) addBlockRectangle(sq,sq,problem);
		for(Integer[] array : group){
			addBlockByArray(array,problem);
		}
		
		return problem;
	}

	public static ProblemContent generateSudoku(int [][] grid)
	{
		int n = grid.length; 
		int block = (int)(Math.sqrt(n) + 1e-10);
		int[] cell = new int[n*n];
		for(int i=0,index=0;i<n;i++)
			for(int j=0;j<n;j++)
			{
				cell[index++] = grid[i][j];
			}
		ProblemContent pc = new ProblemContent(n , cell);
		pc.addBlockVerticalAndHorizonal();
		pc.addBlockRectangle(block, block);
		return pc;
	}
	LinkedList<Integer> toList(String str){
		LinkedList<Integer> list = new LinkedList<Integer>();
		Scanner scan = new Scanner(str);
		while(scan.hasNext()){
			list.add(scan.nextInt());
		}
		return list;
	}
	public static ProblemContent loadXML(String filename)
	{
		ProblemBuilder builder = new ProblemBuilder();
		try{
		    // hLgr_[t@Ng𐶐
		    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
		      // hLgr_[𐶐
		    DocumentBuilder build = dbfactory.newDocumentBuilder();
		      // p[XsDocumentIuWFNg擾
		    Document doc = build.parse(new File(filename));
		    
			Problem problem = new Problem(doc);
			
			Question question = problem.getQuestion();
			Constraint constraint = problem.getConstraint();
			int n ;
			n = builder.numSize = Integer.parseInt(problem.getSize());
			
			// <question>
			
			builder.cell = Utility.toIntArray(question.getContent(), n * n);
			
			// <constraint>
			
			builder.vertical = (constraint.getVertical()==null ||constraint.getVertical().equals("on"));
			
			builder.horizontal = (constraint.getHorizonal()==null ||constraint.getHorizonal().equals("on"));
			
			builder.diagonal = (constraint.getDiagonal()!=null && constraint.getDiagonal().equals("on"));
			
			builder.default_block = (constraint.getDefaultBlock()!=null&&constraint.getDefaultBlock().equals("on"));
			
			// <group> *
			for(Group group : constraint.getGroup())
			{
				Integer[] array = Utility.toIntegerArray(group.getContent(), n * n);
				builder.addGroup(array);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return builder.build();
	}

	public static void main(String[] args) {
		if(true){
			
			ProblemContent pc = ProblemBuilder.loadXML("gen.xml");
			Status result = Solver.answer(new Status(pc) , new SolverMethod());
			result.showDebugData();
		}else{
			try{
				/*Document doc = xml.UJAXP.getValidDocument(
				       new StringReader("84.xml"));*/

			    // hLgr_[t@Ng𐶐
			    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
			      // hLgr_[𐶐
			    DocumentBuilder builder = dbfactory.newDocumentBuilder();
			      // p[XsDocumentIuWFNg擾
			    Document doc = builder.parse(new File("74.xml"));
			    
				jp.gr.puzzle.npv2.xml.Problem problem = new jp.gr.puzzle.npv2.xml.Problem(doc);
				Question question = problem.getQuestion();
				jp.gr.puzzle.npv2.xml.Constraint constraint = problem.getConstraint();
				System.out.println(question);
				System.out.println(constraint.getDefaultBlock());
				System.out.println(constraint.getDiagonal());
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}

}
