/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.core;

import java.util.*;

public class Solver {
	public static enum KindOfAnswer{
		// B or Ȃ or  or s
		UNIQUE_ANSWER , NO_ANSWER , MULTIPLE_ANSWER , IRREGULAR_PROBLEM , NO_JUDGE
	} ;
	
	public static boolean Swordfish(Status state){
		boolean updated = false;
		int sz = state.getSize();
		ArrayList<Integer> pair = new ArrayList<Integer>();
		for(int n=1;n<=sz;n++){
			pair.clear();
			for(int i=0;i<sz+sz;i++){
				if(state.getCandCountOfBlock(i, n)>3) {
					pair.add(-1);
					continue;					
				}
				LinkedList<Integer> list = state.getCellIndexListHavingCandidateNumberInBlock(i, n);
				int bit = 0;
				for(int x : list){
					bit |= (1<<x);
				}
				pair.add(bit);
			}
			// vertical
			
			for(int i=0;i<sz;i++) if(pair.get(i)>0) 
				for(int j=i+1;j<sz;j++) if(pair.get(j)>0)
					for(int k=j+1;k<sz;k++) if(pair.get(k)>0)
					{
						int intersect = pair.get(i) | pair.get(j) | pair.get(k) ;
						if(Bit.getNumberOf1Bit(intersect) != 3) continue;
						int g1=-1,g2=-1,g3=-1;
						//
						for(int g=0;g<state.getSize();g++){
							if((intersect & (1<<g)) != 0){
								g3 = g2; g2 = g1; g1 = g + sz ;
							}
						}
						
						for(int cell_idx : state.getBlock(g1)){
							if(cell_idx%sz == i || cell_idx%sz == j || cell_idx%sz == k) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
						for(int cell_idx : state.getBlock(g2)){
							if(cell_idx%sz == i || cell_idx%sz == j || cell_idx%sz == k) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
						for(int cell_idx : state.getBlock(g3)){
							if(cell_idx%sz == i || cell_idx%sz == j || cell_idx%sz == k) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
					}
			// horizon
			
			for(int i=sz;i<sz+sz;i++) if(pair.get(i)>0) 
				for(int j=i+1;j<sz+sz;j++) if(pair.get(j)>0)
					for(int k=j+1;k<sz+sz;k++) if(pair.get(k)>0)
					{
						int intersect = pair.get(i) | pair.get(j) | pair.get(k) ;
						if(Bit.getNumberOf1Bit(intersect) != 3) continue;
						int g1=-1,g2=-1,g3=-1;
						//
						for(int g=0;g<state.getSize();g++){
							if((intersect & (1<<g)) != 0){
								g3 = g2; g2 = g1; g1 = g ;
							}
						}
						
						for(int cell_idx : state.getBlock(g1)){
							if(cell_idx/sz == i-sz || cell_idx/sz == j-sz || cell_idx/sz == k-sz) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
						for(int cell_idx : state.getBlock(g2)){
							if(cell_idx/sz == i-sz || cell_idx/sz == j-sz || cell_idx/sz == k-sz) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
						for(int cell_idx : state.getBlock(g3)){
							if(cell_idx/sz == i-sz || cell_idx/sz == j-sz || cell_idx/sz == k-sz) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
					}
		}
		
		return updated;
	}
	
	public static boolean XWing(Status state){
		boolean updated = false;
		int sz = state.getSize();
		ArrayList<Integer> pair = new ArrayList<Integer>();
		for(int n=1;n<=sz;n++){
			pair.clear();
			for(int i=0;i<sz+sz;i++){
				if(state.getCandCountOfBlock(i, n)!=2) {
					pair.add(-1);
					continue;					
				}
				LinkedList<Integer> list = state.getCellListHaveCandidateNumberInBlock(i, n);
				int p1 = list.get(0);
				int p2 = list.get(1);
				pair.add(p1*(sz*sz+1)+p2);
			}
			// vertical
			for(int i=0;i<sz;i++) if(pair.get(i)>0) 
				for(int j=i+1;j<sz;j++) if(pair.get(j)>0)
				{
					int cell1_1 = pair.get(i) / (sz*sz+1);
					int cell1_2 = pair.get(i) % (sz*sz+1);
					int cell2_1 = pair.get(j) / (sz*sz+1);
					int cell2_2 = pair.get(j) % (sz*sz+1);
					if(cell1_1/sz > cell1_2 /sz){
						int tmp = cell1_1 ;
						cell1_1 = cell1_2;
						cell1_2 = tmp;
					}
					if(cell2_1/sz > cell2_2 /sz){
						int tmp = cell2_1 ;
						cell2_1 = cell2_2;
						cell2_2 = tmp;
					}
					if(cell1_1/sz == cell2_1/sz && cell1_2/sz == cell2_2/sz){
						int g1 = cell1_1/sz + sz;
						int g2 = cell1_2/sz + sz;
						for(int cell_idx : state.getBlock(g1)){
							if(cell_idx == cell1_1 || cell_idx == cell2_1) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
						for(int cell_idx : state.getBlock(g2)){
							if(cell_idx == cell1_2 || cell_idx == cell2_2) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
					}
				}
			// horizon
			for(int i=sz;i<sz+sz;i++) if(pair.get(i)>0)
				for(int j=i+1;j<sz+sz;j++) if(pair.get(j)>0)
				{
					int cell1_1 = pair.get(i) / (sz*sz+1);
					int cell1_2 = pair.get(i) % (sz*sz+1);
					int cell2_1 = pair.get(j) / (sz*sz+1);
					int cell2_2 = pair.get(j) % (sz*sz+1);
					if(cell1_1%sz > cell1_2%sz){
						int tmp = cell1_1 ;
						cell1_1 = cell1_2;
						cell1_2 = tmp;
					}
					if(cell2_1%sz > cell2_2%sz){
						int tmp = cell2_1 ;
						cell2_1 = cell2_2;
						cell2_2 = tmp;
					}
					if(cell1_1%sz == cell2_1%sz && cell1_2%sz == cell2_2%sz){
						int g1 = cell1_1%sz;
						int g2 = cell1_2%sz;
						for(int cell_idx : state.getBlock(g1)){
							if(cell_idx == cell1_1 || cell_idx == cell2_1) continue;
							if(removeCand(state,cell_idx, n)){
								updated = true;
							}
						}
						for(int cell_idx : state.getBlock(g2)){
							if(cell_idx == cell1_2 || cell_idx == cell2_2) continue;
							if(removeCand(state,cell_idx, n)){
								updated=true;
							}
						}
					}
				}
		}
		return updated;
		
	}
	
	public static boolean nakedTriple(Status state){
		boolean updated = false;

		for(int i=0;i<state.getBlockNum();i++)		
		{
			ArrayList<Integer> id = new ArrayList<Integer>();
			ArrayList<Integer> pair = new ArrayList<Integer>();

			for(int cell_idx : state.getBlock(i))
			{
				if(state.getCandCountOfCell(cell_idx)<=3){
					LinkedList<Integer> list = state.getCandidateList(cell_idx);
					int bit = 0;
					for(int x : list){
						bit |= (1<<x);
					}
					id.add(cell_idx);
					pair.add(bit);
				}
			}
			int m = id.size();
			for(int j=0;j<m;j++)
				for(int k=0;k<j;k++)
					for(int l=0;l<k;l++)
					{
						int intersect = pair.get(j) | pair.get(k) | pair.get(l);
						if(Bit.getNumberOf1Bit(intersect) == 3)
						{
							int p1 = 0;
							int p2 = 0;
							int p3 = 0;
							//
							for(int n=1;n<=state.getSize();n++){
								if((intersect & (1<<n)) != 0){
									p3 = p2; p2 = p1; p1 = n ;
								}
							}
							for(int cell_idx : state.getBlock(i))
							{
								if(id.get(j)==cell_idx || id.get(k)==cell_idx || id.get(l)==cell_idx) continue;

								if(removeCand(state,cell_idx, p1)){
									updated = true;
								}
								if(removeCand(state,cell_idx, p2)){
									updated = true;
								}
								if(removeCand(state,cell_idx, p3)){
									updated = true;
								}
							}
						}
					}
			}
		
		return updated;
	}
	
	public static boolean hiddenTriple(Status state){
		boolean updated = false;
		for(int block_idx=0;block_idx<state.getBlockNum();block_idx++)
		{
			ArrayList<Integer> id = new ArrayList<Integer>();
			ArrayList<Integer> pair = new ArrayList<Integer>();

			for(int n=1;n<=state.getSize();n++){
				if(state.getCandCountOfBlock(block_idx, n)<=3) {
					LinkedList<Integer> list = state.getCellIndexListHavingCandidateNumberInBlock(block_idx, n);
					int bit = 0;
					for(int x : list){
						bit |= (1<<x);
					}
					pair.add(bit);
					id.add(n);
				}
			}
			
			for(int i=0;i<id.size();i++){
				for(int j=0;j<i;j++)
					for(int k=0;k<j;k++)
					{					
						int intersect = pair.get(i) | pair.get(j) | pair.get(k);
						if(Bit.getNumberOf1Bit(intersect) == 3){
							int cell1 = 0;
							int cell2 = 0;
							int cell3 = 0;
							//
							for(int c=0;c<state.getSize();c++){
								if((intersect & (1<<c)) != 0){
									cell3 = cell2; cell2 = cell1; cell1 = state.getBlock(block_idx)[c] ;
								}
							}
							if(state.getCandCountOfCell(cell1)>2){
								for(int n=1;n<=state.getSize();n++){
									if(n==id.get(i) || n==id.get(j) || n==id.get(k)) continue;
									if(removeCand(state,cell1, n)){
										updated = true;
									}
								}
							}
							if(state.getCandCountOfCell(cell2)>2){
								for(int n=1;n<=state.getSize();n++){
									if(n==id.get(i) || n==id.get(j) || n==id.get(k)) continue;
									if(removeCand(state,cell2, n)){
										updated = true;
									}
								}
							}
							if(state.getCandCountOfCell(cell3)>2){
								for(int n=1;n<=state.getSize();n++){
									if(n==id.get(i) || n==id.get(j) || n==id.get(k)) continue;
									if(removeCand(state,cell3, n)){
										updated = true;
									}
								}
							}
						}
					}
			}
		}
		return updated;
	}
	public static boolean localization(Status state){
		boolean updated = false;
		int[] 
			c1  = new int[state.getSize()+1],
			c2  = new int[state.getSize()+1],
			c12 = new int[state.getSize()+1];
		for(Pair<Integer,Integer> p : state.getBlockConstraint().getIntersetionList())
			{
				int i = p.getFirst();
				int j = p.getSecond();
				Integer[] a = state.getBlockInterSection(i, j);
				if(a.length <= 1) continue;
				
				for(int x=1;x<=state.getSize();x++){
					c1[x] = state.getCandCountOfBlock(i, x);
					c2[x] = state.getCandCountOfBlock(j, x);
					c12[x] = 0 ;
				}
				for(int cell_idx : a){
					for(int x=1;x<=state.getSize();x++){
						if(state.isCand(cell_idx, x)){
							c1[x]--; c2[x]--; c12[x]++;
						}
					}
				}
				for(int n=1;n<=state.getSize();n++){
					if(c12[n]==0) continue;
					if(c1[n]>0 && c2[n]==0){
						for(int cell : state.getBlock(i)){
							if(Arrays.binarySearch(a, cell) >= 0) continue;
							if(removeCand(state,cell, n)){
								updated = true; 				
							}
						}
					}else if(c1[n]==0 && c2[n]>0){
						for(int cell : state.getBlock(j)){
							if(Arrays.binarySearch(a, cell) >= 0) continue;
							if(removeCand(state,cell, n)){
								updated = true; 				
							}
						}						
					}
				}
				
			}
		if(updated){
			//state.showDebugData();
		}
		return updated;
	}

	public static boolean nakedPair(Status state){
		boolean updated = false;
		for(int i=0;i<state.getBlockNum();i++)		
		{
			ArrayList<Integer> id = new ArrayList<Integer>();
			ArrayList<Integer> pair = new ArrayList<Integer>();

			for(int cell_idx : state.getBlock(i))
			{
				if(state.getCandCountOfCell(cell_idx)==2){
					LinkedList<Integer> list = state.getCandidateList(cell_idx);
					int p1 = list.get(0);
					int p2 = list.get(1);
					id.add(cell_idx);
					pair.add(p1*(state.getSize()+1)+p2);
				}
			}
			int m = id.size();
			for(int j=0;j<m;j++)
				for(int k=0;k<j;k++)
				{
					if(pair.get(j)==pair.get(k))
					{
						int p1 = pair.get(j)/(state.getSize()+1);
						int p2 = pair.get(j)%(state.getSize()+1);
						for(int cell_idx : state.getBlock(i))
						{
							if(id.get(j)==cell_idx || id.get(k)==cell_idx) continue;
							if(removeCand(state,cell_idx, p1)){
								updated = true;
							}
							if(removeCand(state,cell_idx, p2)){
								updated = true;
							}
						}
					}
				}
		}
		return updated;
	}
	
	public static boolean hiddenPair(Status state){
		boolean updated = false;
		for(int block_idx=0;block_idx<state.getBlockNum();block_idx++)
		{
			ArrayList<Integer> id = new ArrayList<Integer>();
			ArrayList<Integer> pair1 = new ArrayList<Integer>();
			ArrayList<Integer> pair2 = new ArrayList<Integer>();

			for(int n=1;n<=state.getSize();n++){
				if(state.getCandCountOfBlock(block_idx, n)!=2) continue;
				LinkedList<Integer> list = state.getCellListHaveCandidateNumberInBlock(block_idx, n);
				if(list.size() != 2){
					System.out.println("ERRORR");
					continue;
				}
				int p1 = list.get(0);
				int p2 = list.get(1);
				id.add(n);
				pair1.add(p1);
				pair2.add(p2);
			}
			
			for(int i=0;i<id.size();i++){
				for(int j=0;j<i;j++){
					if(pair1.get(i)==pair1.get(j) && pair2.get(i)==pair2.get(j)){
						int cell1 = pair1.get(i);
						int cell2 = pair2.get(i);
						if(state.getCandCountOfCell(cell1)>2){
							for(int n=1;n<=state.getSize();n++){
								if(n==id.get(i) || n==id.get(j)) continue;
								if(removeCand(state,cell1, n)){
									updated = true;
								}
							}
						}
						if(state.getCandCountOfCell(cell2)>2){
							for(int n=1;n<=state.getSize();n++){
								if(n==id.get(i) || n==id.get(j)) continue;
								if(removeCand(state,cell2, n)){
									updated = true;
								}
							}
						}
					}
				}
			}
		}
		return updated;
	}
	static public Status answer(Status state , SolverMethod method) {
		state.unique = method.unique;
		{
			int [] cell = state.getCell();
			for(int i=0;i<cell.length;i++)
				if(cell[i] != 0){
					if(state.isCand(i,cell[i]) == false){
						state.setKindOfAnswer(Solver.KindOfAnswer.IRREGULAR_PROBLEM);
					}
					deleteCandPeer(state, i, cell[i]);
				}
		}
		if(state.isInvalid()){
			return state;
		}
		boolean updated = true;
		while(updated) {
			updated = false;
			if(state.getSpaceCount()==0) break;
			if(method.localization && updated == false) {
				updated = localization(state);
				//if(updated) System.out.println("SOLVE LOCALIZATION");				
			}
			if(method.nakedPair && updated == false) {
				updated = nakedPair(state);
				//if(updated) System.out.println("SOLVE NAMED PAIR");
			}
			if(method.hiddenPair && updated == false) {
				updated = hiddenPair(state);
				//if(updated) System.out.println("SOLVE HIDDEN PAIR");
			}
			if(method.XWing && updated == false){
				updated = XWing(state);
				//if(updated) System.out.println("SOLVE XWing");
			}
			if(method.nakedTriple && updated == false){
				updated = nakedTriple(state);
				//if(updated) System.out.println("SOLVE nakedTriple");
			}

			if(method.hiddenTriple && updated == false){
				updated = hiddenTriple(state);
				//if(updated) System.out.println("SOLVE hiddenTriple");
			}

			if(method.swordfish && updated == false){
				updated = Swordfish(state);
				//if(updated) System.out.println("SOLVE swordfish");
			}
			if(state.isNoAnswer()) return state;
		}
		//System.out.println("Space count : " + state.getSpaceCount());
		if(state.getSpaceCount()>0)
			state.setKindOfAnswer(KindOfAnswer.MULTIPLE_ANSWER);
		else 
			state.setKindOfAnswer(KindOfAnswer.UNIQUE_ANSWER);
		return state; 
	}

	static private Status deleteCandPeer(Status state , int cell_idx , int n){
		for(int x=1;x<=state.getSize();x++){
			if(x != n)
				removeCand(state,cell_idx, x);
		}
		
		for(int block_idx : state.getBlockConstraint().getBlockWhereCellBelong(cell_idx)){
			for(int x : state.getBlock(block_idx)) if(cell_idx != x)
			{
				removeCand(state,x , n);
			}
		}
		return state;
	}
	
	static private boolean removeCand(Status state , int cell_idx , int n){
		if(!state.deleteCandidate(cell_idx, n)) return false ;
		if(state.unique.cellUnique && state.isUniqueCandidate(cell_idx) && state.isEmptyCell(cell_idx)){
			addNumber(state , cell_idx, state.getUniqueCandidate(cell_idx));
			//addNumber(state , cell_idx, Bit.ntz(state.getCell(cell_idx)) + 1);
		}
		BlockConstraint block = state.getBlockConstraint();
		for (int block_idx : block.getBlockWhereCellBelong()[cell_idx])
		{
			if(!state.unique.vhUnique    &&  state.isVHBlock(block_idx)) continue;
			if(!state.unique.blockUnique && !state.isVHBlock(block_idx)) continue;
			if(state.getCandCountOfBlock(block_idx, n)==1){
				for(int c : state.getBlock(block_idx))
					if(state.isCand(c,n)){
						addNumber(state , c, n);
						break;
					}
			}
		}
		return true;
	}
	
	static public boolean addNumber(Status state, int cell_idx , int n)
	{
		if( state.assignValue(cell_idx ,n)) {
			deleteCandPeer(state, cell_idx ,n);
			return true;
		}
		return false;
	}
	
	static public boolean addNumbers( Status state, int[] cells ) {
		boolean ok = true;
		for(int i=0;i<cells.length;i++){
			if(cells[i]>0){
				ok &= Solver.addNumber(state , i, cells[i]);
			}
		}
		return ok;
	}
		
	public static void main(String[] args) {

		ProblemContent p = ProblemBuilder.loadXML("74.xml");
		long starttime = System.currentTimeMillis();
		Status state = new Status(p);
		for(int i=0;i<state.getCellSize();i++){
			if(p.getCell()[i] > 0){
				Solver.addNumber(state, i, p.getCell()[i]);
			}
		}
		SolverMethod sol =  new SolverMethod();
		sol.nakedPair=true;
		sol.localization=true;
		sol.hiddenPair=true;
		for(int i=0;i<1;i++){
			//Solver solver = new Solver(new Status(state.getCell(),state.getSize(),state.getBlockConstraint()));
			//state = solver.answer();
			state = Solver.answer(state,sol);
			//System.out.println("! " + state.getKindOfAnswer());
		}
		long endtime = System.currentTimeMillis();
		System.out.println(" time : " + (endtime - starttime));
		state.showDebugData();
		System.out.println();
		
	}

}
