/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.core;

import java.util.Arrays;
import java.util.LinkedList;

public class Status 
{
	// Zɓ肤鐔̌
	private int numSize ;
	
	// Z@F  ̓pl
	private int [] cell ;
	
	// @̍Wɂ̒l͓邩@[idx][r]
	//private boolean[][] cand ;
	private int[] cand ;

	// ̃ubNɁA݂̐͑邩 [block_idx][n]
	private int[] exist ;

	// ̃ubŃA̐肤}X̌ [block_idx][n]
	private int[][] candCountOfBlock ;

	// 󂫃}X̐
	private int spaceCount;

	// ubNɂ鐧
	private BlockConstraint block;

	private int candCount ;
	
	private Solver.KindOfAnswer kind ;
	
	UniqueMethod unique = new UniqueMethod();
	
	final public boolean isCand(int cell_idx , int n){
		return (cand[cell_idx] & (1<<(n-1))) != 0;
	}
	final public Solver.KindOfAnswer getKindOfAnswer(){
		return kind;
	}
	final public void setKindOfAnswer(Solver.KindOfAnswer kind){
		this.kind = kind;
	}
	final public void setUniqueMethod(UniqueMethod m){
		unique = m; 
	}
	final public boolean isInvalid(){
		return  kind == Solver.KindOfAnswer.NO_ANSWER ||
				kind == Solver.KindOfAnswer.IRREGULAR_PROBLEM;
	}
	// set get
	final public int[] getCell()     { return cell; }
	final public int getCell(int idx){ return cell[idx] ; }
	final public boolean isEmptyCell(int idx) { return cell[idx]==0; }
	final public int getCellSize(){ return cell.length; }
	final public int getSize() { return numSize ; }
	final public int getBlockNum() { return block.getBlock().size(); }
	final public int getCandCountOfCell(int cell_idx){
		return Bit.getNumberOf1Bit(cand[cell_idx]);
	}
	final public int getCandCount(){
		return candCount;
	}
	final public boolean isNoCandidate(int cell_idx){
		return cand[cell_idx]==0;
	}
	final public boolean isUniqueCandidate(int cell_idx){
		return !isNoCandidate(cell_idx) && Bit.Right1BitOff(cand[cell_idx]) == 0;
	}
	final public int getUniqueCandidate(int cell_idx){
		return Bit.ntz(cand[cell_idx])+1;
	}
	final public BlockConstraint getBlockConstraint(){
		return block;
	}
	final public int getCandCountOfBlock(int block_idx , int n){
		return candCountOfBlock[block_idx][n];
	}
	final public Integer[] getBlock(int block_idx){ return block.getBlock().get(block_idx); }
	final public boolean isExistNumberOnZone(int idx , int x){ return (exist[idx]&(1<<(x-1)))!=0; }
	final public int getSpaceCount(){ return spaceCount; }

	public boolean isVHBlock(int block_idx){
		return (block_idx < 2 * numSize) ;
	}
	/**
	 * Z(+1)Ԗڂ̌̐Ԃ
	 * @param cell_idx
	 * @param n
	 * @return ԍ
	 */
	public int getNthCandOfCell(int cell_idx , int n){
		int x = cand[cell_idx];
		while(n>0){
			x = Bit.Right1BitOff(x); n--;
		}
		if(x==0) return -1;
		return Bit.ntz(x) + 1;
	}

	public Status(int num , BlockConstraint block)
	{
		int n2 = num * num ;
		this.cell            = new int    [n2];
		this.block           = block;
		this.numSize         = num ;
		cand                 = new int    [n2] ;
		candCountOfBlock     = new int    [block.getBlockSize()][num + 1];
		exist                = new int    [block.getBlockSize()];
		clear();
	}
	public Status(ProblemContent p)
	{		
		this(p.getNumberSize() , new BlockConstraint(p.getBlock() ,p.getNumberSize()));
	}
	//public static long copytime = 0 , count = 0; 
	public void copyStatusToThis(Status state)
	{
		//count ++ ; 
		//long pt = System.nanoTime();
		this.unique               = state.unique;
		this.block			      = state.block;
		this.kind                 = state.kind;
		this.numSize              = state.numSize;
		this.spaceCount           = state.spaceCount;
		for(int i=0;i<candCountOfBlock.length;i++){
			System.arraycopy(state.candCountOfBlock[i], 0, candCountOfBlock[i], 0, candCountOfBlock[i].length);
			//candCountOfBlock[i] = state.candCountOfBlock[i].clone();
		}
		System.arraycopy(state.cell , 0, cell , 0, cell .length);
		System.arraycopy(state.cand , 0, cand , 0, cand .length);
		System.arraycopy(state.exist, 0, exist, 0, exist.length);
		//cell = state.cell.clone();
		//cand = state.cand.clone();
		//exist = state.exist.clone();
		//copytime += System.nanoTime() - pt ;
		
	}
	public void clear(){
		kind = Solver.KindOfAnswer.NO_JUDGE;
		Arrays.fill(cell, 0);
		Arrays.fill(exist, 0);
		spaceCount = cell.length;
		candCount = numSize * numSize * numSize;
		for(int i=0;i<candCountOfBlock.length;i++){
			Arrays.fill(candCountOfBlock[i], numSize) ; 
		}
		Arrays.fill(cand , (1<<numSize)-1);
	}
	/**
	 * ZՖʂɕs邩ǂׂ
	 * @return
	 */
	public boolean isValid(){
		for(Integer[] b : block.getBlock()) {
			int used = 0 ;
			for(int x : b){
				if((used & (1<<cell[x])) != 0) return false;
				used |= (1<<cell[x]);
			}
		}
		return true;
	}
	/*int GetNumberOfCandidateOfCell(int cell_idx){
		return candCountOfCell[cell_idx];
	}*/
	/**
	 * Zɓ̔ԍXgŕԂ
	 * @param cell_idx Zԍ
	 * @return
	 */
	LinkedList<Integer> getCandidateList(int cell_idx){
		LinkedList<Integer> list = new LinkedList<Integer>();
		for(int i=1;i<=numSize;i++)
			if(isCand(cell_idx,i)) 
				list.add(i);
		return list;
	}
	/**
	 * ̃ubNnƂĂZԍXgƂĕԂ
	 * @param block_idx
	 * @param n
	 * @return
	 */
	LinkedList<Integer> getCellListHaveCandidateNumberInBlock(int block_idx , int n)
	{
		LinkedList<Integer> list = new LinkedList<Integer>();
		for(int cell_idx : getBlock(block_idx)){
			if(isCand(cell_idx,n)==true){
				list.add(cell_idx);
			}
		}
		return list;
	}
	/**
	 * ̃ubNnƂĂZ̃O[vIndexXgƂĕԂ
	 * @param block_idx
	 * @param n
	 * @return
	 */
	LinkedList<Integer> getCellIndexListHavingCandidateNumberInBlock(int block_idx , int n)
	{
		LinkedList<Integer> list = new LinkedList<Integer>();
		for(int i=0;i<getBlock(block_idx).length;i++){
			if(isCand(getBlock(block_idx)[i],n)==true){
				list.add(i);
			}
		}
		return list;
	}
	/**
	 * ȂՖʂǂ̔s
	 * @return ȂȂTrue ȊOȂFalse
	 */
	public boolean isNoAnswer(){
		if(kind == Solver.KindOfAnswer.NO_ANSWER) return true;
		if(kind == Solver.KindOfAnswer.IRREGULAR_PROBLEM) return true;
		if(true) return false;
		// TEST CODE
		boolean flg = false; 
		for(int i=0;i<cell.length;i++){
			if(cell[i] == 0 && isNoCandidate(i)){
				flg = true; break;
			}
		}
		if(flg){
			if(kind != Solver.KindOfAnswer.NO_ANSWER &&
			kind != Solver.KindOfAnswer.IRREGULAR_PROBLEM)
				System.out.println("STRANGE");
		}else {
			if(kind == Solver.KindOfAnswer.NO_ANSWER ||
				kind == Solver.KindOfAnswer.IRREGULAR_PROBLEM){
					System.out.println("STRANGE 2 : " + kind );
					//showDebugData();
			}
		}
		return flg;
	}
	Integer[] getBlockInterSection(int a , int b){
		if(a == b) return null;
		if(a > b) return getBlockInterSection(b , a);
		return block.getBlockSetIntesection()[a][b];
	}
	/**
	 * ̃ZɈӂɓ鐔߂
	 * @param cell_idx Zԍ
	 * @return ӂɌ܂ꍇ͂̐A܂Ȃꍇ-1Ԃ
	 */
	int UniqueCandidateNumberOfCell(int cell_idx){
		// ][̐ÃZɓ肤鐔P̏ꍇ

		if(isUniqueCandidate(cell_idx)) {
			return Bit.ntz(cand[cell_idx]) + 1 ;
		}
		// blockł̐肤̂̏ꍇ
		for(int block_idx : block.getBlockWhereCellBelong(cell_idx))
		{
			//if(block_idx < num_size * 2) continue;
			for(int j=1;j<=numSize;j++){
				if(candCountOfBlock[block_idx][j]==1){
					if(isCand(cell_idx,j))
						return j;
				}
			}
		}
		return -1;
	}
	/**
	 * Zɐ蓖Ă
	 * @param cell_idx Zԍ
	 * @param n 蓖Ă鐔
	 */
	boolean assignValue(int cell_idx , int n){
		if(n==0) return false;		
		if(!isEmptyCell(cell_idx)) {
			if( n != cell[cell_idx] ) {
				setKindOfAnswer(Solver.KindOfAnswer.NO_ANSWER); 			
			}
			return false ;
		}
		if(isCand(cell_idx,n)==false) { 
			setKindOfAnswer(Solver.KindOfAnswer.NO_ANSWER); 
			return false; 
		}
		if(isNoAnswer()) return false;
		cell[cell_idx] = n ;
		spaceCount--;
		for(int block_idx : block.getBlockWhereCellBelong(cell_idx))
		{
			if(isExistNumberOnZone(block_idx, n)){
				kind = Solver.KindOfAnswer.NO_ANSWER;				
			}else
				exist[block_idx] |= (1<<(n-1));
		}
		return true;
	}
			
	/**
	 * Ž₩琔𖕏
	 * @param cell_idx@Zԍ
	 * @param n@鐔
	 * @return ȂTrue AłɂĂi܂ϓȂjȂFalse
	 */
	boolean deleteCandidate(int cell_idx , int n)
	{
		if(n==0) return false;
		if(isNoAnswer()) return false;
		if(isCand(cell_idx,n)==false) return false;
		if(isNoCandidate(cell_idx)){
			kind = Solver.KindOfAnswer.NO_ANSWER;
			return false;
		}
		cand[cell_idx] &= ~(1<<(n-1)); 
		candCount--;
		// ̃ZĂ][␔炷
		for (int block_idx : block.getBlockWhereCellBelong()[cell_idx])
		{
			--candCountOfBlock[block_idx][n] ;
			if(candCountOfBlock[block_idx][n]==0){
				setKindOfAnswer(Solver.KindOfAnswer.NO_ANSWER);
			}
		}
		return true;
	}
	void showBlock(){ 
		int index = 0;
		for(Integer[] seq : block.getBlock() )
		{
			System.out.println(index++);
			int[][] field = new int[numSize][numSize];
			for(int x : seq) field[x/numSize][x%numSize] = 1;
			for(int i=0;i<numSize;i++){
				for(int j=0;j<numSize;j++)
					System.out.print(" " + field[i][j]);
				System.out.println("");
			}
			System.out.println("");
		}
	}
	void showCandData(){
		for(int i=0;i<numSize;i++){
			for(int j=0;j<numSize;j++){
				System.out.print(" " + getCandCountOfCell(i*numSize+j) + " ");
				for(int k=1;k<=numSize;k++){
					if(isCand(i*numSize+j,k)){
						System.out.print(k);
					}else
						System.out.print('-');
				}
			}
			System.out.println("");	
		}
	}
	void showCell(){
		Utility.printGrid(cell, numSize);
	}
	public void showDebugData()
	{
		
		System.out.println("! "+getSpaceCount());
		
		for(int i=0;i<block.getBlockSize();i++){
			for(int j=1;j<=numSize;j++)
				System.out.print(" " + candCountOfBlock[i][j]);
			System.out.println("");
		}
		showCandData();
		for(int i=0;i<cell.length;i++){
			System.out.print(" " + getCandCountOfCell(i));
		}
		System.out.println("");	

		for(int i=0;i<numSize;i++){
			for(int j=0;j<numSize;j++){
				System.out.print(" " + cell[i*numSize+j]);
			}
			System.out.println("");
		}			
	}
}
