/*
 *    Copyright(C) 2007 Time Intermedia Corporation <puzzle@timedia.co.jp>
 *    All rights reserved.
 *
 *    Number Place Generator Version 2.0
 *        Director:   Hirofumi Fujiwara
 *        Puzzler:    Naoki Inaba
 *        Programmer: Masaya Kiwada
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *    
 */

package jp.gr.puzzle.npv2.gui;

import java.awt.BorderLayout;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Scanner;
import java.util.concurrent.TimeUnit;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileFilter;

import jp.gr.puzzle.npv2.core.BlockConstraint;
import jp.gr.puzzle.npv2.core.BlockSplit;
import jp.gr.puzzle.npv2.core.Evaluator;
import jp.gr.puzzle.npv2.core.Generator;
import jp.gr.puzzle.npv2.core.NumberPlaceFile;
import jp.gr.puzzle.npv2.core.Pair;
import jp.gr.puzzle.npv2.core.Solver;
import jp.gr.puzzle.npv2.core.SolverMethod;
import jp.gr.puzzle.npv2.core.Status;
import jp.gr.puzzle.npv2.core.Utility;
import jp.gr.puzzle.npv2.gui.Grid.MODE;

public class NPGeneratorV2 extends JFrame {
	private static final long serialVersionUID = 1L;
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		//Solver.Method m = new Solver.Method();
		MainFrame frame = new MainFrame();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.repaint();
	}
}

/*
 * ver 2.0 [2007/12/18] : Number Place Generator 2.0 Formal
 */

class MainFrame extends JFrame{

	private static final long serialVersionUID = 1L;
	MainPanel panel ;
	public MainFrame() {
		Container cont = this.getContentPane();
		panel = new MainPanel();
		panel.getGrid().setMode(Grid.MODE.SETTING);
		initMenu();
		cont.add( panel, "Center" );
		setTitle("Number Place Generator BatchMode [2019-03-23]");
		addCopyright();
		setSize(800 ,600);
		setVisible(true);		
		panel.getFocusOnBoard();
		panel.getGrid().loadFile();
		panel.getEditPanel().doGenerate();
		while(panel.getEditPanel().genThread != null) {System.out.println("Wait");}
		panel.getGrid().saveFile();
		System.exit(0);
	}
	private void addCopyright() {
		JLabel copylabel = new JLabel(
				"Copyright(c)2007  Time Intermedia Corporation. GPL; dead link: http://puzzle.gr.jp/", 
				JLabel.CENTER );
		copylabel.setFont(new Font("Dialog",Font.PLAIN,12));
		add( copylabel, "South");	
	}	
	void initMenu(){
		JMenuBar menubar = new JMenuBar();
		setJMenuBar(menubar);
		JMenu menu = new JMenu("File");
		menubar.add(menu);
		JMenuItem save = new JMenuItem("Save File");
		JMenuItem load = new JMenuItem("Open File");
		menu.add(save);
		menu.add(load);
		save.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					panel.getGrid().saveFile();
				}
			}
		);
		load.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					panel.getGrid().loadFile();
				}
			}
		);
	}
}

/**
 * Solver �̎�؂�ݒ肷��_�C�A���O
 * 
 */
class SolverConfigDialog extends JDialog implements ActionListener
{
	private static final long serialVersionUID = 1L;
	SolverMethod method;
	Pair<Integer , Integer> border;
	JCheckBox localizationCheck = new JCheckBox("Localization");
	JCheckBox nakedPairCheck = new JCheckBox("Naked Pair");
	JCheckBox hiddenPairCheck = new JCheckBox("Hidden Pair");
	JCheckBox nakedTripleCheck = new JCheckBox("Naked Triple");
	JCheckBox hiddenTripleCheck = new JCheckBox("Hidden Triple");
	JCheckBox XWingCheck = new JCheckBox("X Wing");
	JCheckBox SwordfishCheck = new JCheckBox("Swordfish");
	JCheckBox VHUniqueCheck = new JCheckBox("VH Unique");
	JCheckBox BlockUniqueCheck = new JCheckBox("Block Unique");
	JCheckBox CellUniqueCheck = new JCheckBox("Cell Unique");
	JTextField LowerPoint = new JTextField();
	JTextField UpperPoint = new JTextField();
	JButton okButton = new JButton("OK");
	JButton cancelButton = new JButton("Cancel");
	public SolverConfigDialog(SolverMethod method , Pair<Integer,Integer> border) {
		this.method = method;
		this.border = border;
		this.getContentPane().setLayout(new GridLayout(5,1,5,5));
		this.setTitle("specify using procedure");
		JPanel panel1 = new JPanel();
		JPanel panel2 = new JPanel();
		JPanel panel3 = new JPanel();
		JPanel panel4 = new JPanel();
		JPanel panel5 = new JPanel();
		System.out.println(method.localization);
		localizationCheck.setSelected(method.localization);
		nakedPairCheck.setSelected(method.nakedPair);
		hiddenPairCheck.setSelected(method.hiddenPair);
		nakedTripleCheck.setSelected(method.nakedTriple);
		hiddenTripleCheck.setSelected(method.hiddenTriple);
		XWingCheck.setSelected(method.XWing);
		SwordfishCheck.setSelected(method.swordfish);
		VHUniqueCheck.setSelected(method.unique.vhUnique);
		CellUniqueCheck.setSelected(method.unique.cellUnique);
		BlockUniqueCheck.setSelected(method.unique.blockUnique);
		LowerPoint.setText(Integer.toString(border.getFirst()));
		UpperPoint.setText(Integer.toString(border.getSecond()));
		LowerPoint.setColumns(8);
		UpperPoint.setColumns(8);
		panel1.add(BlockUniqueCheck);
		panel1.add(VHUniqueCheck);
		panel1.add(CellUniqueCheck);
		panel2.add(localizationCheck);
		panel2.add(nakedPairCheck);
		panel2.add(hiddenPairCheck);
		panel2.add(XWingCheck);
		panel3.add(nakedTripleCheck);
		panel3.add(hiddenTripleCheck);
		panel3.add(SwordfishCheck);
		panel4.add(new JLabel("DP range : "));
		panel4.add(LowerPoint);
		panel4.add(new JLabel(" - "));
		panel4.add(UpperPoint);
		panel5.add(okButton);
		panel5.add(cancelButton);
		panel1.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		panel2.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		panel3.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		panel4.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		panel5.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		this.getContentPane().add(panel1);
		this.getContentPane().add(panel2);
		this.getContentPane().add(panel3);
		this.getContentPane().add(panel4);
		this.getContentPane().add(panel5);
		this.setModal(true);
		this.pack();
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
	}
	public int getLowerPoint(){
		try{
			return Math.max(Integer.parseInt(LowerPoint.getText()) , 0);
		}catch(Exception e){
			return 0 ;
		}
	}
	public int getUpperPoint(){
		try{
			int v = Integer.parseInt(UpperPoint.getText());
			if(v < 0) v = Integer.MAX_VALUE;
			return v;
		}catch(Exception e){
			return Integer.MAX_VALUE;
		}
	}
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == okButton){
			this.method.localization = localizationCheck.isSelected();
			this.method.nakedPair  = nakedPairCheck.isSelected();
			this.method.hiddenPair = hiddenPairCheck.isSelected();
			this.method.nakedTriple= nakedTripleCheck.isSelected();
			this.method.hiddenTriple= hiddenTripleCheck.isSelected();
			this.method.XWing= XWingCheck.isSelected();
			this.method.swordfish= SwordfishCheck.isSelected();
			this.method.unique.vhUnique = VHUniqueCheck.isSelected();
			this.method.unique.cellUnique = CellUniqueCheck.isSelected();
			this.method.unique.blockUnique = BlockUniqueCheck.isSelected();
			this.border.setFirst(getLowerPoint());
			this.border.setSecond(getUpperPoint());
			if(this.border.getFirst() > this.border.getSecond()){
				int t = this.border.getFirst() ;
				this.border.setFirst(this.border.getSecond());
				this.border.setSecond(t);
			}
			System.out.println(localizationCheck.isSelected());
			System.out.println(nakedTripleCheck.isSelected());
			System.out.println(hiddenTripleCheck.isSelected());
			System.out.println(SwordfishCheck.isSelected());
			this.dispose();
		}else if(e.getSource() == cancelButton){
			this.dispose();
		}
	}
}

class ProblemConfigPanel extends JPanel implements ActionListener
{
	private static final long serialVersionUID = 5603102565528079155L;
	private JComboBox numSizeBox = new JComboBox(); ; 
	private JComboBox rectBlockConstraintBox = new JComboBox();;
	private MainPanel mainPanel;
	private JCheckBox diagonalCheckbox = new JCheckBox("diagonal");
	JButton lineEditButton = new JButton("Line Edit");
	private JButton randomButton   = new JButton("Random");
	private JButton solverConfigButton = new JButton("Solver Config");
	private JComboBox forbiddenBox = new JComboBox();
	private JLabel hintLabel = new JLabel("Hint : 0");
	public ProblemConfigPanel(MainPanel mainPanel) {
		this.mainPanel = mainPanel;
		setupControl();
	}
	private Grid getGrid(){
		return mainPanel.getGrid();
	}
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==rectBlockConstraintBox) {
			changeRectBlock();
		}
		else if(e.getSource()==diagonalCheckbox){       
			getGrid().setDiagonal(diagonalCheckbox.isSelected());
			getGrid().setMode(Grid.MODE.SETTING);
		}
		else if(e.getSource()==numSizeBox){
			changeNumSize((Integer)numSizeBox.getSelectedItem());
			setSquareBlockConstraint();
		}
		else if(e.getSource()==lineEditButton){
			if(getGrid().getMode()!=Grid.MODE.LINE_EDIT){
				setLineEditMode();
			}
			else{
				unsetLineEditMode();
			}
		}
		else if(e.getSource()==solverConfigButton){
			Pair<Integer , Integer> p = new Pair<Integer, Integer>(
				mainPanel.getGrid().lower_point , mainPanel.getGrid().upper_point	
				);
			SolverConfigDialog scd = new SolverConfigDialog(getGrid().method, p);
			scd.setVisible(true);
			scd.requestFocus();
			mainPanel.getGrid().lower_point = p.getFirst();
			mainPanel.getGrid().upper_point = p.getSecond();
		}
		else if(e.getSource()==randomButton){
			splitBlock();
		}
		getGrid().requestFocus();
		getGrid().repaint();
	}
	final static int LOWER = 2 , UPPER = 25;
	public void setNumSize(int n){
		numSizeBox.setSelectedIndex(n - LOWER);
		setupRectBlockConstraintBox(n);
	}
	void splitBlock(){
		BlockSplit bs = new BlockSplit(getGrid().getNumSize() , getGrid().getNumSize());
		int[] bb = bs.splitBlock();
		getGrid().setLineDelimiter(bb);		
	}
	private void setupForbiddenBox(int n){
		forbiddenBox.removeAllItems();
		forbiddenBox.addItem("none");
		for(int i=1;i<=n;i++){
			forbiddenBox.addItem(i);
		}
		forbiddenBox.setSelectedIndex(0);
	}
	private void setupControl(){
		for(int i=LOWER;i<=UPPER;i++){
			numSizeBox.addItem(i);
		}		
		this.add(hintLabel);		
		/*
		loadButton.addActionListener(this);
		this.add(loadButton);
		
		saveButton.addActionListener(this);
		this.add(saveButton);
		*/

		numSizeBox.setSelectedItem(mainPanel.DEFAULT_SIZE);
		numSizeBox.addActionListener(this);
		this.add(new JLabel("size"));
		this.add(numSizeBox);
		
		rectBlockConstraintBox.addActionListener(this);
		setupRectBlockConstraintBox(mainPanel.DEFAULT_SIZE);
		setSquareBlockConstraint();
		changeRectBlock();
		this.add(new JLabel("block"));
		this.add(rectBlockConstraintBox);
		
		lineEditButton.addActionListener(this);
		this.add(lineEditButton);
		lineEditButton.setBackground(Color.white);
		lineEditButton.setVisible(false);

		randomButton.addActionListener(this);
		this.add(randomButton);
		randomButton.setVisible(false);

		forbiddenBox.addActionListener(this);
		setupForbiddenBox(mainPanel.DEFAULT_SIZE);
		this.add(new JLabel("forbidden"));
		this.add(forbiddenBox);
		
		diagonalCheckbox.addActionListener(this);
		this.add(diagonalCheckbox);		

		solverConfigButton.addActionListener(this);
		this.add(solverConfigButton);
	}
	public void setHintNumber(int n){
		hintLabel.setText("Hint : " + n);
	}
	private void setupRectBlockConstraintBox(int n){
		rectBlockConstraintBox.removeAllItems();
		//rectBlockConstraintBox.addItem("none");
		rectBlockConstraintBox.addItem("specify");
		rectBlockConstraintBox.addItem("random");
		for(int i=2;i<n;i++){
			if(n%i != 0) continue;
			int j = n/i;
			if(i>j) break;
			String str = i + " �~ " + j ;
			if(i==j) {
				str += " *";
			}
			rectBlockConstraintBox.addItem(str);
		}
	}
	private void setSquareBlockConstraint(){
		String s = (String) rectBlockConstraintBox.getItemAt(rectBlockConstraintBox.getItemCount()-1);
		if(s.endsWith("*"))
			rectBlockConstraintBox.setSelectedIndex(rectBlockConstraintBox.getItemCount()-1);
	}
	Pair<Integer , Integer> parseString(String s){
		//if(s.equals("none")) return new Pair(-1,-1);
		if(s.equals("specify")) return new Pair<Integer , Integer>(-1,-1);
		if(s.equals("random"))    return new Pair<Integer , Integer>(-1,-1);
		Scanner sc = new Scanner(s);
		int col    = sc.nextInt() ; 
		sc.next();
		int row    = sc.nextInt() ;
		return new Pair<Integer , Integer>(row,col);
	}
	private void changeNumSize(int n){
		if(n==getGrid().getNumSize()) return ;
		setupRectBlockConstraintBox(n);
		setupForbiddenBox(n);
		getGrid().setNumSize(n);
		changeRectBlock();

	}
	private void setLineEditMode(){
		getGrid().clearFixTable();		
		getGrid().setMode(Grid.MODE.LINE_EDIT);
		lineEditButton.setBackground(Color.green);		
	}
	private void unsetLineEditMode(){
		getGrid().setMode(Grid.MODE.SETTING);
		lineEditButton.setBackground(Color.white);
	}
	private void changeRectBlock(){
		String s = (String)rectBlockConstraintBox.getSelectedItem();
		if(s == null) return ;
		if(s.equals("specify")) {
			lineEditButton.setVisible(true);
			randomButton.setVisible(false);
		}else if(s.equals("random")){
			randomButton.setVisible(true);
			lineEditButton.setVisible(false);
			unsetLineEditMode();			
			splitBlock();
			return ;
		}else{
			randomButton.setVisible(false);
			lineEditButton.setVisible(false);
			unsetLineEditMode();			
		}
		Pair<Integer , Integer> p = parseString(s);
		getGrid().setRectBlock(p.getFirst(), p.getSecond());
	}
	public void setEnabled(boolean b){
		/*saveButton.setEnabled(b); 
		loadButton.setEnabled(b);*/ 
		numSizeBox.setEnabled(b); 
		rectBlockConstraintBox.setEnabled(b); 
		diagonalCheckbox.setEnabled(b); 
		lineEditButton .setEnabled(b); 
		randomButton.setEnabled(b);
	}
	public int getSpecifiedForbiddenNumber(){
		int val = forbiddenBox.getSelectedIndex();
		if(val==0) val=-1;
		return val;
	}
}

class EditButtonPanel extends JPanel implements ActionListener 
{
	private static final long serialVersionUID = 1L;
	JButton		clearAllButton;		// �S�N���A
	JButton		clearWhiteButton;	// ���}�X�N���A
	JButton		clearNumberButton;	// �����N���A
	private	JLabel		modeLabel;			// ���݂̃��[�h��\
	JButton		setButton;			// Set      �{�^��
	JButton		execButton;			// Generate �{�^��
	private	boolean		stopPushed;
	JButton		playButton;			// Play     �{�^��
	JButton		problemButton;		// Problem  �{�^��
	JButton		answerButton;		// Problem  �{�^��
	JButton		solveButton;		// Solve�@�@�{�^��
	JButton		autoButton;			// Auto �@�@�{�^��
	public  Thread		genThread;
	private JLabel      difficultyLabel ; 
	private MainPanel mainPanel;
	private int lastDifficult = 0 ;
	private	Generator	gen;
	
	public EditButtonPanel(MainPanel mainPanel) {
		this.mainPanel = mainPanel;
		setupButton();
		setDoubleBuffered(true);
	}
	public int getLastDifficult(){
		return lastDifficult;	
	}
	private void setupButton(){
		clearAllButton = new JButton("Clear All");
		clearWhiteButton = new JButton("Clear White");
		clearNumberButton = new JButton("Clear Number");
		setButton = new JButton("Set");
		execButton = new JButton("Generate");	
		playButton = new JButton("Play");
		problemButton = new JButton("Problem");
		answerButton = new JButton("Answer");
		solveButton = new JButton("Solve");
		autoButton =  new JButton("Auto");
		modeLabel  = new JLabel("SET");
		difficultyLabel = new JLabel("");

		{
			clearAllButton.setEnabled(true);
			clearWhiteButton.setEnabled(true);
			clearNumberButton.setEnabled(true);
			execButton.setEnabled(true);	
			setButton.setEnabled(true);
			playButton.setEnabled(false);
			solveButton.setEnabled(true);
			problemButton.setEnabled(false);
			answerButton.setEnabled(false);	
		}
		
		JPanel clearpanel = new JPanel();
		JPanel editpanel = new JPanel();

		this.setLayout(new BorderLayout());
		this.add(clearpanel,BorderLayout.SOUTH);
		this.add(editpanel,BorderLayout.NORTH);

		JSeparator separator  = new JSeparator(SwingConstants.VERTICAL);
		separator.setPreferredSize(new Dimension(10 , 30));
		JSeparator separator2 = new JSeparator(SwingConstants.VERTICAL);
		separator2.setPreferredSize(new Dimension(10 , 30));
		clearpanel.add(clearAllButton);
		clearpanel.add(clearNumberButton);
		clearpanel.add(separator2);
		clearpanel.add(clearWhiteButton);
		
		editpanel.setLayout(new FlowLayout());
		editpanel.add(modeLabel);
		editpanel.add(setButton);
		editpanel.add(execButton);
		editpanel.add(solveButton);
		editpanel.add(separator);
		editpanel.add(playButton);
		editpanel.add(problemButton);
		editpanel.add(answerButton);
		editpanel.add(difficultyLabel);
		//editpanel.add(autoButton);
		
		clearAllButton.addActionListener(this);
		clearWhiteButton.addActionListener(this);
		clearNumberButton.addActionListener(this);
		setButton.addActionListener(this);
		execButton.addActionListener(this);	
		playButton.addActionListener(this);
		problemButton.addActionListener(this);
		answerButton.addActionListener(this);
		solveButton.addActionListener(this);
		autoButton.addActionListener(this);
		
	}
	public void setDifficultyPoint(int n , BlockConstraint block , int[] problem){
		double ev = Evaluator.evaluate(n, block, problem);
		lastDifficult = (int)ev ; 
		difficultyLabel.setText(lastDifficult + "pt"); 
		difficultyLabel.repaint();
		// System.out.println("EVAL : " + ev);
	}
	public JLabel getModeLabel(){
		return modeLabel;
	}
	public JButton getPlayButton(){
		return playButton;
	}
	public void doGenerate() {
			if(genThread != null){
				stopPushed = true;
				if( gen != null )
					gen.setInterruption(true);
			}else{
				stopPushed = false;
				execButton.setText("     Stop    ");
				getGrid().clearAnswer();
				getGrid().setMode(Grid.MODE.GENERATOR);
				mainPanel.configPanel.setEnabled(false);
				mainPanel.editPanel.clearAllButton.setEnabled(false);
				mainPanel.editPanel.clearWhiteButton.setEnabled(false);
				mainPanel.editPanel.clearNumberButton.setEnabled(false);
				mainPanel.editPanel.execButton.setEnabled(true);	
				mainPanel.editPanel.setButton.setEnabled(false);
				mainPanel.editPanel.playButton.setEnabled(false);
				mainPanel.editPanel.solveButton.setEnabled(false);
				mainPanel.editPanel.problemButton.setEnabled(false);
				mainPanel.editPanel.answerButton.setEnabled(false);		
				genThread = new Thread() {
					public void run() {
						if(execution()) {
							getGrid().repaint();	
							mainPanel.editPanel.clearAllButton.setEnabled(false);
							mainPanel.editPanel.clearWhiteButton.setEnabled(false);
							mainPanel.editPanel.clearNumberButton.setEnabled(false);
							mainPanel.editPanel.execButton.setEnabled(true);	
							mainPanel.editPanel.setButton.setEnabled(true);
							mainPanel.editPanel.playButton.setEnabled(true);
							mainPanel.editPanel.solveButton.setEnabled(false);
							mainPanel.editPanel.problemButton.setEnabled(false);
							mainPanel.editPanel.answerButton.setEnabled(false);	
						}else{
							getGrid().setMode(Grid.MODE.SETTING);				
						}
						mainPanel.configPanel.setEnabled(true);	
						execButton.setText("Generate");
						genThread = null;
					}
				};
				genThread.start();
			}
			getGrid().repaint();
		}
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == execButton){
			doGenerate();
		}
	    else if(e.getSource() == solveButton){
			if(getGrid().getMode()==Grid.MODE.SETTING){
				solve(getGrid().getHidden(),getGrid().method);
			}
		}else if(e.getSource() == clearAllButton){
			getGrid().clearAll();
		}else if(e.getSource() == clearNumberButton){
			getGrid().clearNumber();
		}else if(e.getSource() == clearWhiteButton){
			getGrid().clearWhite();
		}else if(e.getSource() == setButton){
			modeLabel.setText("SET");
			getGrid().setMode(Grid.MODE.SETTING);
		}else if(e.getSource() == playButton){
			modeLabel.setText("PLAY");
			getGrid().setMode(Grid.MODE.PLAY);
		}else if(e.getSource() == answerButton){
			getGrid().setMode(Grid.MODE.ANSWER);
		}else if(e.getSource() == problemButton){
			getGrid().setMode(Grid.MODE.PROBLEM);
		}
		mainPanel.getFocusOnBoard();
	}
	private boolean execution() {
		long	starttime = System.currentTimeMillis();
		int loopcount = 0;
		int[] result = null;
		gen = mainPanel.getGrid().makeGenerator();
		if(gen == null) {
			return false;
		}
		gen.setForbidden(mainPanel.getConfigPanel().getSpecifiedForbiddenNumber());
		System.out.println("GENERATE ======================");
		modeLabel.setText( "" + 0 + " times / " + 0 + " msec" );
		//Status.copytime = 0 ; Status.count = 0 ;
		
		BlockConstraint blockConst = getGrid().makeBlockConstraint() ;
		double maxv = 0.0 ;
		while(result == null) {
			if( stopPushed ) {
				genThread = null;
				return false;
			}
			result = gen.generate();
			if(gen.getSeed() != null) ++loopcount;
			long current = System.currentTimeMillis();
			long elapsedtime = current - starttime;
			modeLabel.setText( "" + loopcount + " times / " + elapsedtime + " msec" );
			modeLabel.repaint();
			if(result != null){
				double v = Evaluator.evaluate(getGrid().getNumSize(), blockConst, result);
				setDifficultyPoint(getGrid().getNumSize() , getGrid().makeBlockConstraint() , result);
				if(v < getGrid().lower_point || getGrid().upper_point < v)
					result = null ;
				if(false && v > maxv){
					maxv = v ;
					System.out.print("score : " + maxv) ; 
					int sz = getGrid().getNumSize();
					for(int i=0;i<sz;i++){
						for(int j=0;j<sz;j++){
							System.out.print(" " + result[i*sz+j]);
						}
						System.out.println();
					}
				}
				//result = null;
			}
			
		}
		
		//System.out.println("copy time : " + Status.copytime + " " + Status.count);
		setDifficultyPoint(getGrid().getNumSize() , getGrid().makeBlockConstraint() , result);
		System.arraycopy(result, 0, getGrid().getProblem(), 0, result.length);
		//		Evaluator.debug=true;
		SolverMethod method = new SolverMethod();
		method.setAllUse();
		solve(result , method); 
		getGrid().setAnswer(result);
		getGrid().setupPlayTable();
		return true;
	}
	public Grid getGrid(){ return mainPanel.getGrid(); }
	public boolean solve(int [] cell , SolverMethod method){
		BlockConstraint block = mainPanel.getGrid().makeBlockConstraint();
		if(block==null){
			JOptionPane.showMessageDialog(
					this , "invalid block status" , "Error : block status" ,
					JOptionPane.ERROR_MESSAGE
				);
			return false ; 
		}
		Utility.printGrid(cell, getGrid().getNumSize());
		Status state = new Status(getGrid().getNumSize(),block);
		state.setUniqueMethod(method.unique);
		/*Solver solve = new Solver(
				new Status(cell,getGrid().getNumSize(),block));*/
		for(int i=0;i<cell.length;i++){
			if(cell[i]>0){
				Solver.addNumber(state , i, cell[i]);
			}
		}
		Status result = Solver.answer(state , method);
		System.out.println(result.getKindOfAnswer());
		if(result.isNoAnswer()){
			JOptionPane.showMessageDialog(
					this , "invalid board status" , "Error : invalid problem" ,
					JOptionPane.ERROR_MESSAGE
				);
			return false; 
		}
		System.arraycopy(result.getCell(), 0, cell, 0, cell.length);
		getGrid().repaint();
		return true;
	}
}

class MainPanel extends JPanel
{	
	private static final long serialVersionUID = -1686368919866303841L;
	public final int DEFAULT_SIZE = 9;
	ProblemConfigPanel configPanel;
	EditButtonPanel editPanel;
	Grid grid;
	public MainPanel(){
		grid = new Grid(DEFAULT_SIZE , this);
		editPanel = new EditButtonPanel(this);
		configPanel = new ProblemConfigPanel(this);
		this.setLayout(new BorderLayout());
		this.add(configPanel,BorderLayout.NORTH);
		this.add(grid,BorderLayout.CENTER);
		this.add(editPanel,BorderLayout.SOUTH);
		grid.requestFocus();
	}
	public void getFocusOnBoard() {
		grid.requestFocus();
	}
	ProblemConfigPanel getConfigPanel(){ return configPanel; } 
	EditButtonPanel    getEditPanel()  { return editPanel ;  }
	Grid getGrid(){
		return grid;
	}
}

class Grid extends JPanel 
	implements MouseListener , MouseMotionListener , KeyListener , MouseWheelListener
{	
	private static final long serialVersionUID = 1L;
	private static int boldLineSize = 5;
	private static final int normalLineSize = 1;
	private static final Color markColor = new Color(0xc0,0xde,0xff);
	
	private int numSize;
	private int[] problem , answer , hidden , playtbl;
	private boolean[] hint , blockMark;
	private boolean isDiagonal = false;
	private boolean[][] hLine , vLine ;
	private boolean[] fixed;
	private boolean cursorMoved=false;
	private Point cursor = new Point(0,0);
	private MainPanel mainPanel;
	int lower_point = 1, upper_point = Integer.MAX_VALUE ; 
	SolverMethod method = new SolverMethod();
	public static enum MODE{
		INIT , 
		SETTING    ,
		LINE_EDIT ,
		GENERATOR ,
		PLAY , 
		PROBLEM , 
		ANSWER
	} ;
	MODE mode = MODE.SETTING;
	public void setMode(MODE mode){
		if(this.mode == mode) return ;
		MODE oldmode = this.mode ; 
		this.mode = mode;
		if(oldmode == MODE.LINE_EDIT)
			mainPanel.configPanel.lineEditButton.setBackground(Color.white);
		if(this.mode == MODE.SETTING || this.mode == MODE.LINE_EDIT){
			mainPanel.getEditPanel().getModeLabel().setText("SET");
			mainPanel.configPanel.setEnabled(true);
			mainPanel.editPanel.clearAllButton.setEnabled(true);
			mainPanel.editPanel.clearWhiteButton.setEnabled(true);
			mainPanel.editPanel.clearNumberButton.setEnabled(true);
			mainPanel.editPanel.execButton.setEnabled(true);	
			mainPanel.editPanel.setButton.setEnabled(true);
			mainPanel.editPanel.playButton.setEnabled(false);
			mainPanel.editPanel.solveButton.setEnabled(true);
			mainPanel.editPanel.problemButton.setEnabled(false);
			mainPanel.editPanel.answerButton.setEnabled(false);	
			Arrays.fill(answer, 0);
			Arrays.fill(problem, 0);
			Arrays.fill(playtbl, 0);
		}else if(this.mode == MODE.PLAY){
			mainPanel.getEditPanel().getModeLabel().setText("PLAY");
			mainPanel.configPanel.setEnabled(true);
			mainPanel.editPanel.clearAllButton.setEnabled(false);
			mainPanel.editPanel.clearWhiteButton.setEnabled(true);
			mainPanel.editPanel.clearNumberButton.setEnabled(false);
			mainPanel.editPanel.execButton.setEnabled(false);	
			mainPanel.editPanel.setButton.setEnabled(true);
			mainPanel.editPanel.playButton.setEnabled(true);
			mainPanel.editPanel.solveButton.setEnabled(false);
			mainPanel.editPanel.problemButton.setEnabled(true);
			mainPanel.editPanel.answerButton.setEnabled(true);		
		}else if(this.mode == MODE.GENERATOR){
			//
			mainPanel.editPanel.solveButton.setEnabled(false);
		}else if(this.mode == MODE.ANSWER){
			mainPanel.getEditPanel().getModeLabel().setText("ANSWER");
			mainPanel.configPanel.setEnabled(true);
			mainPanel.editPanel.clearAllButton.setEnabled(false);
			mainPanel.editPanel.clearWhiteButton.setEnabled(false);
			mainPanel.editPanel.clearNumberButton.setEnabled(false);
			mainPanel.editPanel.execButton.setEnabled(false);	
			mainPanel.editPanel.setButton.setEnabled(false);
			mainPanel.editPanel.playButton.setEnabled(true);
			mainPanel.editPanel.solveButton.setEnabled(false);
			mainPanel.editPanel.problemButton.setEnabled(true);
			mainPanel.editPanel.answerButton.setEnabled(true);		
		}else if(this.mode == MODE.PROBLEM){
			mainPanel.getEditPanel().getModeLabel().setText("PROBLEM");
			mainPanel.configPanel.setEnabled(true);
			mainPanel.editPanel.clearAllButton.setEnabled(false);
			mainPanel.editPanel.clearWhiteButton.setEnabled(false);
			mainPanel.editPanel.clearNumberButton.setEnabled(false);
			mainPanel.editPanel.execButton.setEnabled(false);	
			mainPanel.editPanel.setButton.setEnabled(false);
			mainPanel.editPanel.playButton.setEnabled(true);
			mainPanel.editPanel.solveButton.setEnabled(false);
			mainPanel.editPanel.problemButton.setEnabled(true);
			mainPanel.editPanel.answerButton.setEnabled(true);	
		}
		repaint();
	}
	public MODE getMode(){
		return mode;
	}
	private int toXLineIndex(int x){
		int unit = Math.min(getWidth(),getHeight()) / (numSize + 1);
		int x0 = (getWidth()  - unit*numSize) / 2;
		if((x-x0+3)<0) return -1;
		int xi = (x-x0+3)/unit;
		int xj = (x-x0+3)%unit;
		if(xi<numSize && xj <= 7) return xi-1;
		return -1;
	}
	private int toYLineIndex(int y){
		int unit = Math.min(getWidth(),getHeight()) / (numSize + 1);
		int y0 = (getHeight()  - unit*numSize) / 2;
		if((y-y0+3) < 0) return -1;
		int yi = (y-y0+3)/unit;
		int yj = (y-y0+3)%unit;
		if(yi<numSize && yj <= 7) return yi-1;
		return -1;
	}
	private int toXIndex(int x){
		int unit = Math.min(getWidth(),getHeight()) / (numSize + 1);
		int x0 = (getWidth()  - unit*numSize) / 2;
		if(x < x0) return -1;
		int xi = (x-x0)/unit;
		if(xi>=numSize)	return -1;
		return xi;
	}
	private int toYIndex(int y){
		int unit = Math.min(getWidth(),getHeight()) / (numSize + 1);
		int y0 = (getHeight()  - unit*numSize) / 2;
		if(y < y0) return -1;
		int yi = (y-y0)/unit;
		if(yi>=numSize)	return -1;
		return yi;
	}
	private void cursorMove( int dx, int dy ) {
		int x = (cursor.x + numSize + dx) % numSize;
		int y = (cursor.y + numSize + dy) % numSize;	
		cursorMoveTo( x, y );
	}
	public void cursorMoveTo(int x , int y){
		cursor.x = x;
		cursor.y = y;
		cursorMoved = true;
		repaint();
	}
	public Grid(int n , MainPanel mainPanel) {
		setup(n);
		addMouseListener(this);
		addMouseMotionListener(this);
		addMouseWheelListener(this);
		addKeyListener(this);		
		this.mainPanel = mainPanel; 
	}
	private void setLineFromBlockMark(){
		for(int i=0;i<blockMark.length;i++){
			int xi = i % numSize;
			int yi = i / numSize;
			if(xi+1<numSize){
				boolean b1 = blockMark[toIndex(xi, yi)];
				boolean b2 = blockMark[toIndex(xi+1, yi)];
				if(b1 && b2)      vLine[yi][xi]=false;
				else if(b1 != b2) vLine[yi][xi]=true;
			}

			if(yi+1<numSize){
				boolean b1 = blockMark[toIndex(xi, yi)];
				boolean b2 = blockMark[toIndex(xi, yi+1)];
				if(b1 && b2)      hLine[yi][xi]=false;
				else if(b1 != b2) hLine[yi][xi]=true;
			}
		}
		Arrays.fill(blockMark , false);
		repaint();
	}
	
	private int cursorIndex(){
		return toIndex(cursor.x , cursor.y);
	}
	private int toIndex(int x , int y){
		return y * numSize + x;
	}
	public void setup(int n){
		setNumSize(n);
		setRectBlock(-1,-1);
	}
	public void setDiagonal(boolean f){
		this.isDiagonal = f;
	}
	public boolean isDiagonal() { return isDiagonal ; }
	public void setRectBlock(int row ,int col){
		for(int i=0;i<vLine.length;i++) {
			Arrays.fill(vLine[i], false);
		}
		for(int i=0;i<hLine.length;i++) {
			Arrays.fill(hLine[i], false);
		}
		if(row<0 || col<0) return ;
		for(int i = row ; i < numSize ; i += row){
			for(int j=0;j<numSize;j++)
				vLine[j][i-1] = true;
		}
		for(int i = col ; i < numSize ; i += col){
			for(int j=0;j<numSize;j++)
				hLine[i-1][j] = true;
		}
	}
	public int getNumSize(){
		return numSize;
	}
	public void setNumSize(int n){
		this.numSize = n;
		problem     = new int[n*n];
		answer 		= new int[n*n];
		hidden  	= new int[n*n];
		playtbl		= new int[n*n];
		hint   		= new boolean[n*n];
		hLine  		= new boolean[n-1][n]; // [i][j] -> the line between cell[i][j] and cell[i][j+1]
		vLine  		= new boolean[n][n-1]; // [i][j] -> the line between cell[i][j] and cell[i+1][j]
		blockMark 	= new boolean[n*n];
		fixed  		= new boolean[n*n];		
		setHintCount();
		setMode(MODE.SETTING);
	}
	@Override
	public void paint(Graphics g) {
		super.paint(g);
		g.setColor(Color.white);
		g.fillRect(0, 0, getWidth(), getHeight());
		printGrid(g);
	}
	private void printGrid(Graphics g){
		g.setColor( Color.black);
		int size = numSize;
		int unit = Math.min(getWidth(),getHeight()) / (size + 1);
		if( numSize > 16 ) {
			boldLineSize = 3;
		} else if( numSize > 9 ) {
			boldLineSize = 4;
		} else {
			boldLineSize = 5;
		}
		int x0 = (getWidth()  - unit*size) / 2;
		int y0 = (getHeight() - unit*size) / 2;
		if(isDiagonal){
			g.setColor(Color.gray);
			g.drawLine(x0, y0, x0 + unit*size, y0 + unit*size);
			g.drawLine(x0, y0 + unit*size, x0 + unit*size, y0);
		}
		int cur = cursorIndex();
		Font font = new Font( "Dialog", Font.BOLD, unit*4/6 );
		g.setFont(font);
		FontMetrics fm = getFontMetrics( g.getFont() );
		for(int i=0;i<hint.length;i++){
			int xi = i % size;
			int yi = i / size;
			int xp = x0+xi*unit;
			int yp = y0+yi*unit;
			if(mode == MODE.LINE_EDIT && blockMark[i]){
				g.setColor(Color.lightGray);
				g.fillRect(xp, yp, unit, unit);				
			}
			else if(mode != MODE.ANSWER && mode != MODE.PROBLEM && mode != MODE.PLAY && hint[i]){
				g.setColor(markColor);
				g.fillRect(xp, yp, unit, unit);
			}
			int val ;
			if(mode == MODE.ANSWER)
				val = answer[i];
			else if(mode == MODE.PROBLEM)
				val = problem[i];
			else if(mode == MODE.GENERATOR)
				val = answer[i];
			else if(mode == MODE.PLAY)
				val = playtbl[i];
			else
				val = hidden[i];
			if(val!=0){
				String str = Integer.toString(val);
				int xx = xp + (unit - fm.stringWidth(str))/2;
				int yy = yp + (unit - fm.getHeight())/2 + fm.getAscent();
				if(mode == MODE.GENERATOR)
					g.setColor(Color.blue);
				else if(mode == MODE.PLAY || mode == MODE.PROBLEM || mode == MODE.ANSWER){
					if(problem[i]!=0)
						g.setColor(Color.black);
					else
						g.setColor(Color.gray);						
				}
				else
					g.setColor(hint[i]?Color.black:Color.gray);
				g.drawString( str, xx, yy );	
			}
			if(i==cur){
				int w = boldLineSize;
				g.setColor(Color.red);
				g.fillRect(xp , yp , w, unit);		
				g.fillRect(xp , yp , unit, w);		
				g.fillRect(xp+unit-w,yp, w, unit);	
				g.fillRect(xp,yp+unit-w, unit,w);						
			}
		}
		g.setColor( Color.black);
		
		for(int i=0;i<size;i++){
			for(int j=0;j<=size;j++){
				int w ; 
				w = (j==0||j==size||vLine[i][j-1]) ? boldLineSize : normalLineSize;
				g.fillRect( x0 + unit * j-w/2, y0 + unit * i-w/2, w, unit+w);	// �c��
				w = (j==0||j==size||hLine[j-1][i]) ? boldLineSize : normalLineSize;
				g.fillRect( x0 + unit * i-w/2, y0 + unit * j-w/2, unit+w, w);	// ����
			}
		}
	}
	
	@Override
	public void update(Graphics g) {
		super.update(g);
		paint(g);
	}
	
	public void setLineDelimiter(int[] blockArray)
	{
		for(int i=0;i<blockArray.length;i++){
			int xi = i % numSize;
			int yi = i / numSize;
			if(xi+1<numSize){
				int b1 = blockArray[toIndex(xi, yi)];
				int b2 = blockArray[toIndex(xi+1, yi)];
				if(b1 == b2)      vLine[yi][xi]=false;
				else              vLine[yi][xi]=true;
			}

			if(yi+1<numSize){
				int b1 = blockArray[toIndex(xi, yi)];
				int b2 = blockArray[toIndex(xi, yi+1)];
				if(b1 == b2)      hLine[yi][xi]=false;
				else 			  hLine[yi][xi]=true;
			}
		}
		setMode(MODE.SETTING);
		repaint();
	}
	
	public int[] getBlockArray() {
		return BlockConstraint.getBlockArrayFromLine(numSize, vLine, hLine);
	}
	public BlockConstraint makeBlockConstraint(){
		ArrayList<Integer[]> b = new ArrayList<Integer[]>();
		Utility.addBlockVertical(b, numSize);
		Utility.addBlockHorizontal(b, numSize);
		if(isDiagonal)
			Utility.addBlockDiagonal(b, numSize);
		int[] array  = getBlockArray();
		if(array == null) return null;
		Integer[] integer_array = new Integer[array.length];
		for(int i=0;i<array.length;i++)
			integer_array[i] = array[i];
		Utility.addBlockByArray(integer_array, b, numSize);
		return new BlockConstraint(b , numSize);
	}
	public void setAnswer(int[] cell){
		this.answer = cell;
	}
	public int[] getAnswer()
	{
		return answer;
	}
	public void setProblem(int [] p){
		this.problem = p;
	}
	public int[] getProblem(){
		return problem;
	}
	public int[] getHidden(){
		return hidden;
	}
	public void setHidden(int[] cell){
		this.hidden = cell; 
	}
	public Generator makeGenerator(){
		int n2 = numSize*numSize;
		int[] hints   = new int[n2] ;
		for(int i=0;i<numSize;i++){
			for(int j=0;j<numSize;j++){
				int idx = i*numSize + j;
				hints[idx] = this.hint[idx]?1:0;
			}
		}
		BlockConstraint block = makeBlockConstraint();
		if(block == null) {
			JOptionPane.showMessageDialog(
					this , "invalid block status" , "Error : block status" ,
					JOptionPane.ERROR_MESSAGE
				);
			setMode(Grid.MODE.SETTING);
			return null;
		}
		
		if(!mainPanel.getEditPanel().solve(hidden.clone() , this.method)) {
			return null ; 
		}
		Generator generator = new Generator(numSize , hints , hidden , block);
		generator.setMethod(this.method);
		return generator;
	}
	void setupPlayTable(){
		for(int i=0;i<playtbl.length;i++)
			playtbl[i] = problem[i];
	}
	public void clearAll(){
		Arrays.fill(problem, 0);
		Arrays.fill(answer, 0);
		Arrays.fill(hidden   , 0);
		Arrays.fill(hint  , false);
		setHintCount();
		setMode(MODE.SETTING);
		repaint();
	}
	public void clearWhite(){
		if(mode==MODE.SETTING || mode==MODE.LINE_EDIT){
			for(int i=0;i<hidden.length;i++){
				if(!hint[i]) hidden[i]=0;
			}
		}else if(mode==MODE.PLAY){
			for(int i=0;i<playtbl.length;i++){
				if(problem[i]==0) playtbl[i]=0;
			}
		}
		setHintCount();
		repaint();
	}
	public void clearNumber(){
		Arrays.fill(hidden, 0);
		setMode(MODE.SETTING);
		repaint();
	}
	public void clearAnswer(){
		Arrays.fill(answer, 0);
		repaint();
	}
	public void clearFixTable(){
		Arrays.fill(fixed,false);
		Arrays.fill(blockMark,false);
		repaint();
	}
	/*
	 * MOUSE EVENT
	 */
	
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	public void mouseMoved(MouseEvent arg0) {}
	public void mousePressed( MouseEvent e ) {
		requestFocus();
		if(mode == MODE.GENERATOR) return ;
		int xi = toXIndex(e.getX());
		int yi = toYIndex(e.getY());
		if(mode == MODE.LINE_EDIT){
			int xl = toXLineIndex(e.getX());
			int yl = toYLineIndex(e.getY());
			if(xl!=-1 && yl==-1 && yi != -1){
				if((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) 
					vLine[yi][xl]^=true;
			}else if(yl!=-1 && xl==-1 && xi != -1){
				if((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) 
					hLine[yl][xi]^=true;
			}else if( xi != -1 && yi != -1){
				cursorMoveTo(xi, yi);
				if((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0){
					int idx = toIndex(xi, yi); 
					blockMark[idx] ^= true;
					fixed[idx] = true;
				}
			}
			repaint(); return ;
		}
		if( xi<0 || yi<0 )
			return;
		cursorMoveTo(xi, yi);
		if(mode == MODE.SETTING){
			if((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0){
				hint[toIndex(xi, yi)] ^= true;
				setHintCount();
			}
		}
		repaint();
	}
	private void setHintCount(){
		if(mainPanel == null) return ;
		int cnt = 0;
		for(boolean a : hint){
			if(a) cnt++;
		}
		mainPanel.getConfigPanel().setHintNumber(cnt);
		repaint();
	}
	public void mouseReleased(MouseEvent arg0) {
		if(mode == MODE.LINE_EDIT){
			Arrays.fill(fixed,false);
		}
	}
	
	public void mouseClicked(MouseEvent e) {
		if(mode == MODE.LINE_EDIT){
			if(e.getClickCount()==2){ // double clicked
				if((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0){
					setLineFromBlockMark();
				}							
			}
		}
	}
	public void mouseDragged(MouseEvent e) {
		if(mode == MODE.LINE_EDIT){
			int xi = toXIndex(e.getX());
			int yi = toYIndex(e.getY());
			if(xi!=-1 && yi!=-1){
				if((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0){
					int idx = toIndex(xi, yi);
					if(fixed[idx]) return ;
					fixed[idx] = true;
					blockMark[toIndex(xi, yi)] ^= true;
				}							
			}
			repaint();
		}
	}
	
	public void mouseWheelMoved(MouseWheelEvent e) {
		if(mode == MODE.SETTING || mode == MODE.LINE_EDIT){
			int r = e.getWheelRotation();
			int ci = cursorIndex();
			hidden[ci] = (hidden[ci] - r + numSize+1) % (numSize+1);
			repaint();
		}
		else if(mode == MODE.PLAY){
			int r = e.getWheelRotation();
			int ci = cursorIndex();
			if(problem[ci]==0){ 
				playtbl[ci] = (playtbl[ci] - r + numSize+1) % (numSize+1);
				repaint();
			}
		}
	}
	/*
	 * KEY EVENT
	 */
	public void keyReleased(KeyEvent arg0) {}
	public void keyTyped(KeyEvent arg0) {}
	public void keyPressed( KeyEvent e ) {
		if(mode == MODE.GENERATOR) return ;
		switch( e.getKeyCode() ) {
		case KeyEvent.VK_LEFT:
			cursorMove( -1, 0 );	return;
		case KeyEvent.VK_UP:
			cursorMove( 0, -1 );	return;
		case KeyEvent.VK_RIGHT:
			cursorMove( +1, 0 );	return;
		case KeyEvent.VK_DOWN:
			cursorMove( 0, +1 );	return;
		}
		
		char c =  e.getKeyChar();
		int ci = cursorIndex();
		if(c == ' ' || (Character.isDigit(c))){
			int x = c - '0';
			if(c==' ') x = 0 ; 
			int[] target = null;
			if(mode == MODE.LINE_EDIT || mode == MODE.SETTING)
				target = hidden;
			else if(mode == MODE.PLAY && problem[ci]==0)
				target = playtbl;
			if(target != null){
				if(cursorMoved){
					target[ci] = x;							
				}else{
					int nval = target[ci] * 10 + x;
					if(nval > numSize){
						target[ci] = x;								
					}else{
						target[ci] = nval;									
					}
				}
			}
		}else if(c == 'h') {
			if( mode == MODE.SETTING ){
				hint[ci] ^= true;
				setHintCount();
			}
		}
		cursorMoved = false;
		repaint();
	}
	
	private void setDifficultyPoint(){
		mainPanel.getEditPanel().setDifficultyPoint(getNumSize(), makeBlockConstraint(), problem);
	}
	private static FileFilter filter = new FileFilter(){
		@Override
		public boolean accept(File file) {
			if(file.isDirectory()) return true;
			if(file.isFile())
				return file.getName().endsWith("xml");
			return false;
		}
		@Override
		public String getDescription() {
			return "xml file (*.xml)" ;
		}
	};
	private JFileChooser fileDialog = null;
	public void loadFile(){
/*		if(fileDialog==null){
			fileDialog = new JFileChooser(new File("."));
			fileDialog.addChoosableFileFilter(filter);
		}
		if (fileDialog.showOpenDialog(this) == JFileChooser.CANCEL_OPTION) return ;
		File file = fileDialog.getSelectedFile();*/
		File file = new File("in.xml");
		try{
			NumberPlaceFile npFile = new NumberPlaceFile();
			npFile.Load(file);
			setNumSize(npFile.getNumSize());
			this.hint   = npFile.getHint(); 
			this.hidden = npFile.getHidden();
			this.answer = npFile.getAnswer();
			this.problem = npFile.getProblem();
			this.setDiagonal(npFile.isDiagonal());
			boolean default_block = npFile.isDefaultBlock();
		    if(default_block){
		    	int sq = Utility.sqrt(numSize) ;
		    	setRectBlock(sq, sq);
		    }else{
				setLineDelimiter(npFile.getBlockArray());
		    }
			setHintCount();
		    setDifficultyPoint();
		    if(!Utility.isAllZero(problem)){
		    	setupPlayTable();
		    	setMode(MODE.PLAY);
		    }
			/*
		    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
		    DocumentBuilder builder = dbfactory.newDocumentBuilder();
		    Document doc = builder.parse(file);
		    Problem problem = new Problem(doc);
		    Constraint constraint = problem.getConstraint();
		    setNumSize(Integer.parseInt(problem.getSize()));
		    mainPanel.configPanel.setNumSize(numSize);		    
		    this.hint     = Utility.toBooleanArray(problem.getHint()    , numSize * numSize);
		    this.hidden   = Utility.toIntArray    (problem.getHidden()  , numSize * numSize);
		    this.playtbl  = Utility.toIntArray    (null                 , numSize * numSize);
		    this.answer   = Utility.toIntArray    (problem.getAnswer()  , numSize * numSize);
		    this.problem  = Utility.toIntArray    (problem.getQuestion().getContent(), numSize * numSize);
		    this.setDiagonal(
		    		constraint.getDiagonal() != null &&
		    		constraint.getDiagonal().equals(Constraint.DIAGONAL_ON));
		    boolean default_block 
		    	= (constraint.getDefaultBlock()!=null&&constraint.getDefaultBlock().equals("on"));
		    if(default_block){
		    	int sq = Utility.sqrt(numSize) ;
		    	setRectBlock(sq, sq);
		    }else{
		    	for(Group group : constraint.getGroup())
				{
					int[] array = Utility.toIntArray(group.getContent(),numSize*numSize);
					setLineDelimiter(array);
					break;
				}
		    }
			setHintCount();
		    setDifficultyPoint();
		    */
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	public void saveFile(){
/*		if(fileDialog==null){
			fileDialog = new JFileChooser(new File("."));
			fileDialog.addChoosableFileFilter(filter);
		}
		if (fileDialog.showSaveDialog(this) == JFileChooser.CANCEL_OPTION) return ;
		String filename = fileDialog.getSelectedFile().toString();*/
		String filename = "out.xml";
		if(!filename.endsWith(".xml")) filename = filename + ".xml";
		File file = new File (filename);
		if(file.exists()){
			int response = JOptionPane.showConfirmDialog(null,
                    "File already exists. Overwrite?", "Warning", JOptionPane.YES_NO_OPTION);
			
			if(response != JOptionPane.OK_OPTION) return ;
		}		
		System.out.println(file.toString());
		try{
			int[] blockArray = getBlockArray();
			if(blockArray==null)
		    {
				JOptionPane.showMessageDialog(
						this , "invalid block" , "Error : block status" ,
						JOptionPane.ERROR_MESSAGE
					);
				return ;
		    }
			NumberPlaceFile npFile = new NumberPlaceFile();
			npFile.setNumSize(this.numSize);
			npFile.setHint(this.hint);
			npFile.setHidden(this.hidden);
			npFile.setAnswer(this.answer);
			npFile.setDifficult(mainPanel.getEditPanel().getLastDifficult());
			npFile.setIsDiagonal(this.isDiagonal);
			npFile.setBlockArray(blockArray);
			npFile.setProblem(this.problem);
			npFile.Save(file);
			/*
		    Problem problem = new Problem();
		    problem.setSize(Integer.toString(numSize));
		    problem.setName("Number Place");
		    problem.setAuthor("Number Place Generator");
		    problem.setAnswer(Utility.toStringFromArray(this.answer));
		    Constraint constraint = new Constraint();
		    constraint.setDiagonal(isDiagonal?Constraint.DIAGONAL_ON:Constraint.DIAGONAL_OFF);
		    Group group = new Group();
		    Question q = new Question();
		    Integer[] blockArray = getBlockArray();
		    if(blockArray==null)
		    {
				JOptionPane.showMessageDialog(
						this , "invalid block" , "Error : block status" ,
						JOptionPane.ERROR_MESSAGE
					);
				return ;
		    }
		    group.setContent(Utility.toStringFromArray(blockArray));
		    group.setBlock(Group.BLOCK_ON);
		    constraint.setGroup(group);
		    problem.setQuestion(q);
		    problem.getQuestion().setContent(Utility.toStringFromArray(this.problem));
		    problem.getQuestion().setDifficult(Integer.toString(mainPanel.getEditPanel().getLastDifficult()));
		    ///problem.setQuestion(Utility.toStringFromArray(this.problem));
		    problem.setAnswer(Utility.toStringFromArray(this.answer));
		    problem.setHint(Utility.toStringFromArray(this.hint));
		    problem.setHidden(Utility.toStringFromArray(this.hidden));
		    problem.setConstraint(constraint);
		    PrintWriter pw = null;
		    try{
		    	pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));	    	
		    	pw.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
		    	problem.makeTextElement(pw);
		    }catch (Exception e){
		    	e.printStackTrace();
		    }finally{
		    	if(pw != null) pw.close();
		    }*/
		}catch(Exception e){
			e.printStackTrace();
		}
	}
}

