/*
 * The Relaxer artifact
 * Copyright (c) 2000-2003, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package jp.gr.puzzle.npv2.xml;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>Constraint</b> is generated from sudoku.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="constraint">
 *   <optional>
 *     <attribute name="horizonal">
 *       <choice>
 *         <value type="string">on</value>
 *         <value type="string">off</value>
 *       </choice>
 *     </attribute>
 *   </optional>
 *   <optional>
 *     <attribute name="vertical">
 *       <choice>
 *         <value type="string">on</value>
 *         <value type="string">off</value>
 *       </choice>
 *     </attribute>
 *   </optional>
 *   <optional>
 *     <attribute name="default-block">
 *       <choice>
 *         <value type="string">on</value>
 *         <value type="string">off</value>
 *       </choice>
 *     </attribute>
 *   </optional>
 *   <optional>
 *     <attribute name="diagonal">
 *       <choice>
 *         <value type="string">off</value>
 *         <value type="string">on</value>
 *       </choice>
 *     </attribute>
 *   </optional>
 *   <zeroOrMore>
 *     <ref name="Group"/>
 *   </zeroOrMore>
 * </element>
 * -->
 * <!-- for javadoc -->
 * <pre> &lt;element name="constraint"&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="horizonal"&gt;
 *       &lt;choice&gt;
 *         &lt;value type="string"&gt;on&lt;/value&gt;
 *         &lt;value type="string"&gt;off&lt;/value&gt;
 *       &lt;/choice&gt;
 *     &lt;/attribute&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="vertical"&gt;
 *       &lt;choice&gt;
 *         &lt;value type="string"&gt;on&lt;/value&gt;
 *         &lt;value type="string"&gt;off&lt;/value&gt;
 *       &lt;/choice&gt;
 *     &lt;/attribute&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="default-block"&gt;
 *       &lt;choice&gt;
 *         &lt;value type="string"&gt;on&lt;/value&gt;
 *         &lt;value type="string"&gt;off&lt;/value&gt;
 *       &lt;/choice&gt;
 *     &lt;/attribute&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="diagonal"&gt;
 *       &lt;choice&gt;
 *         &lt;value type="string"&gt;off&lt;/value&gt;
 *         &lt;value type="string"&gt;on&lt;/value&gt;
 *       &lt;/choice&gt;
 *     &lt;/attribute&gt;
 *   &lt;/optional&gt;
 *   &lt;zeroOrMore&gt;
 *     &lt;ref name="Group"/&gt;
 *   &lt;/zeroOrMore&gt;
 * &lt;/element&gt;
 * </pre>
 *
 * @version sudoku.rng (Wed Nov 07 13:28:19 JST 2007)
 * @author  Relaxer 1.0 (http://www.relaxer.org)
 */
public class Constraint implements java.io.Serializable, Cloneable {
    public static final String HORIZONAL_ON = "on";
    public static final String HORIZONAL_OFF = "off";
    public static final String VERTICAL_ON = "on";
    public static final String VERTICAL_OFF = "off";
    public static final String DEFAULTBLOCK_ON = "on";
    public static final String DEFAULTBLOCK_OFF = "off";
    public static final String DIAGONAL_OFF = "off";
    public static final String DIAGONAL_ON = "on";

    private String horizonal_;
    private String vertical_;
    private String defaultBlock_;
    private String diagonal_;
    // List<Group>
    private java.util.List group_ = new java.util.ArrayList();

    /**
     * Creates a <code>Constraint</code>.
     *
     */
    public Constraint() {
    }

    /**
     * Creates a <code>Constraint</code>.
     *
     * @param source
     */
    public Constraint(Constraint source) {
        setup(source);
    }

    /**
     * Creates a <code>Constraint</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Constraint(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Constraint</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Constraint(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Constraint</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Constraint(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Constraint</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Constraint(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Constraint</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Constraint(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Constraint</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Constraint(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Constraint</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Constraint(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Constraint</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Constraint(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Constraint</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Constraint(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Constraint</code> by the Constraint <code>source</code>.
     *
     * @param source
     */
    public void setup(Constraint source) {
        int size;
        setHorizonal(source.getHorizonal());
        setVertical(source.getVertical());
        setDefaultBlock(source.getDefaultBlock());
        setDiagonal(source.getDiagonal());
        this.group_.clear();
        size = source.group_.size();
        for (int i = 0;i < size;i++) {
            addGroup((Group)source.getGroup(i).clone());
        }
    }

    /**
     * Initializes the <code>Constraint</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Constraint</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Constraint</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        horizonal_ = URelaxer.getAttributePropertyAsString(element, "horizonal");
        vertical_ = URelaxer.getAttributePropertyAsString(element, "vertical");
        defaultBlock_ = URelaxer.getAttributePropertyAsString(element, "default-block");
        diagonal_ = URelaxer.getAttributePropertyAsString(element, "diagonal");
        group_.clear();
        while (true) {
            if (Group.isMatch(stack)) {
                addGroup(new Group(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Constraint(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("constraint");
        int size;
        if (this.horizonal_ != null) {
            URelaxer.setAttributePropertyByString(element, "horizonal", this.horizonal_);
        }
        if (this.vertical_ != null) {
            URelaxer.setAttributePropertyByString(element, "vertical", this.vertical_);
        }
        if (this.defaultBlock_ != null) {
            URelaxer.setAttributePropertyByString(element, "default-block", this.defaultBlock_);
        }
        if (this.diagonal_ != null) {
            URelaxer.setAttributePropertyByString(element, "diagonal", this.diagonal_);
        }
        size = this.group_.size();
        for (int i = 0;i < size;i++) {
            Group value = (Group)this.group_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Constraint</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Constraint</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Constraint</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Constraint</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Constraint</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Constraint</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>horizonal</b>.
     *
     * @return String
     */
    public final String getHorizonal() {
        return (horizonal_);
    }

    /**
     * Sets the String property <b>horizonal</b>.
     *
     * @param horizonal
     */
    public final void setHorizonal(String horizonal) {
        this.horizonal_ = horizonal;
    }

    /**
     * Gets the String property <b>vertical</b>.
     *
     * @return String
     */
    public final String getVertical() {
        return (vertical_);
    }

    /**
     * Sets the String property <b>vertical</b>.
     *
     * @param vertical
     */
    public final void setVertical(String vertical) {
        this.vertical_ = vertical;
    }

    /**
     * Gets the String property <b>defaultBlock</b>.
     *
     * @return String
     */
    public final String getDefaultBlock() {
        return (defaultBlock_);
    }

    /**
     * Sets the String property <b>defaultBlock</b>.
     *
     * @param defaultBlock
     */
    public final void setDefaultBlock(String defaultBlock) {
        this.defaultBlock_ = defaultBlock;
    }

    /**
     * Gets the String property <b>diagonal</b>.
     *
     * @return String
     */
    public final String getDiagonal() {
        return (diagonal_);
    }

    /**
     * Sets the String property <b>diagonal</b>.
     *
     * @param diagonal
     */
    public final void setDiagonal(String diagonal) {
        this.diagonal_ = diagonal;
    }

    /**
     * Gets the Group property <b>Group</b>.
     *
     * @return Group[]
     */
    public final Group[] getGroup() {
        Group[] array = new Group[group_.size()];
        return ((Group[])group_.toArray(array));
    }

    /**
     * Sets the Group property <b>Group</b>.
     *
     * @param group
     */
    public final void setGroup(Group[] group) {
        this.group_.clear();
        for (int i = 0;i < group.length;i++) {
            addGroup(group[i]);
        }
    }

    /**
     * Sets the Group property <b>Group</b>.
     *
     * @param group
     */
    public final void setGroup(Group group) {
        this.group_.clear();
        addGroup(group);
    }

    /**
     * Adds the Group property <b>Group</b>.
     *
     * @param group
     */
    public final void addGroup(Group group) {
        this.group_.add(group);
    }

    /**
     * Adds the Group property <b>Group</b>.
     *
     * @param group
     */
    public final void addGroup(Group[] group) {
        for (int i = 0;i < group.length;i++) {
            addGroup(group[i]);
        }
    }

    /**
     * Gets number of the Group property <b>Group</b>.
     *
     * @return int
     */
    public final int sizeGroup() {
        return (group_.size());
    }

    /**
     * Gets the Group property <b>Group</b> by index.
     *
     * @param index
     * @return Group
     */
    public final Group getGroup(int index) {
        return ((Group)group_.get(index));
    }

    /**
     * Sets the Group property <b>Group</b> by index.
     *
     * @param index
     * @param group
     */
    public final void setGroup(int index, Group group) {
        this.group_.set(index, group);
    }

    /**
     * Adds the Group property <b>Group</b> by index.
     *
     * @param index
     * @param group
     */
    public final void addGroup(int index, Group group) {
        this.group_.add(index, group);
    }

    /**
     * Remove the Group property <b>Group</b> by index.
     *
     * @param index
     */
    public final void removeGroup(int index) {
        this.group_.remove(index);
    }

    /**
     * Remove the Group property <b>Group</b> by object.
     *
     * @param group
     */
    public final void removeGroup(Group group) {
        this.group_.remove(group);
    }

    /**
     * Clear the Group property <b>Group</b>.
     *
     */
    public final void clearGroup() {
        this.group_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<constraint");
        if (horizonal_ != null) {
            buffer.append(" horizonal=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getHorizonal())));
            buffer.append("\"");
        }
        if (vertical_ != null) {
            buffer.append(" vertical=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getVertical())));
            buffer.append("\"");
        }
        if (defaultBlock_ != null) {
            buffer.append(" default-block=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getDefaultBlock())));
            buffer.append("\"");
        }
        if (diagonal_ != null) {
            buffer.append(" diagonal=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getDiagonal())));
            buffer.append("\"");
        }
        buffer.append(">");
        size = this.group_.size();
        for (int i = 0;i < size;i++) {
            Group value = (Group)this.group_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</constraint>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<constraint");
        if (horizonal_ != null) {
            buffer.write(" horizonal=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getHorizonal())));
            buffer.write("\"");
        }
        if (vertical_ != null) {
            buffer.write(" vertical=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getVertical())));
            buffer.write("\"");
        }
        if (defaultBlock_ != null) {
            buffer.write(" default-block=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getDefaultBlock())));
            buffer.write("\"");
        }
        if (diagonal_ != null) {
            buffer.write(" diagonal=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getDiagonal())));
            buffer.write("\"");
        }
        buffer.write(">");
        size = this.group_.size();
        for (int i = 0;i < size;i++) {
            Group value = (Group)this.group_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</constraint>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<constraint");
        if (horizonal_ != null) {
            buffer.print(" horizonal=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getHorizonal())));
            buffer.print("\"");
        }
        if (vertical_ != null) {
            buffer.print(" vertical=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getVertical())));
            buffer.print("\"");
        }
        if (defaultBlock_ != null) {
            buffer.print(" default-block=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getDefaultBlock())));
            buffer.print("\"");
        }
        if (diagonal_ != null) {
            buffer.print(" diagonal=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getDiagonal())));
            buffer.print("\"");
        }
        buffer.print(">");
        size = this.group_.size();
        for (int i = 0;i < size;i++) {
            Group value = (Group)this.group_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</constraint>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHorizonalAsString() {
        return (URelaxer.getString(getHorizonal()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getVerticalAsString() {
        return (URelaxer.getString(getVertical()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDefaultBlockAsString() {
        return (URelaxer.getString(getDefaultBlock()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDiagonalAsString() {
        return (URelaxer.getString(getDiagonal()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHorizonalByString(String string) {
        setHorizonal(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setVerticalByString(String string) {
        setVertical(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDefaultBlockByString(String string) {
        setDefaultBlock(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDiagonalByString(String string) {
        setDiagonal(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Constraint</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "constraint")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        while (true) {
            if (!Group.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Constraint</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Constraint</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
