/*
 * The Relaxer artifact
 * Copyright (c) 2000-2003, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package jp.gr.puzzle.npv2.xml;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>Problem</b> is generated from sudoku.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="problem">
 *   <attribute name="size"/>
 *   <attribute name="name"/>
 *   <attribute name="author"/>
 *   <optional>
 *     <ref name="Question"/>
 *   </optional>
 *   <ref name="Constraint"/>
 *   <optional>
 *     <ref name="Answer"/>
 *   </optional>
 *   <optional>
 *     <ref name="Hint"/>
 *   </optional>
 *   <optional>
 *     <ref name="Hidden"/>
 *   </optional>
 *   <optional>
 *     <ref name="Seed"/>
 *   </optional>
 *   <optional>
 *     <ref name="Comment"/>
 *   </optional>
 * </element>
 * -->
 * <!-- for javadoc -->
 * <pre> &lt;element name="problem"&gt;
 *   &lt;attribute name="size"/&gt;
 *   &lt;attribute name="name"/&gt;
 *   &lt;attribute name="author"/&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Question"/&gt;
 *   &lt;/optional&gt;
 *   &lt;ref name="Constraint"/&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Answer"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Hint"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Hidden"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Seed"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Comment"/&gt;
 *   &lt;/optional&gt;
 * &lt;/element&gt;
 * </pre>
 *
 * @version sudoku.rng (Wed Nov 07 13:28:19 JST 2007)
 * @author  Relaxer 1.0 (http://www.relaxer.org)
 */
public class Problem implements java.io.Serializable, Cloneable {
    private String size_;
    private String name_;
    private String author_;
    private Question question_;
    private Constraint constraint_;
    private String answer_;
    private String hint_;
    private String hidden_;
    private String seed_;
    private String comment_;

    /**
     * Creates a <code>Problem</code>.
     *
     */
    public Problem() {
        size_ = "";
        name_ = "";
        author_ = "";
    }

    /**
     * Creates a <code>Problem</code>.
     *
     * @param source
     */
    public Problem(Problem source) {
        setup(source);
    }

    /**
     * Creates a <code>Problem</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Problem(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Problem</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Problem(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Problem</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Problem(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Problem</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Problem</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Problem</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Problem</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Problem</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Problem</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Problem</code> by the Problem <code>source</code>.
     *
     * @param source
     */
    public void setup(Problem source) {
        int size;
        setSize(source.getSize());
        setName(source.getName());
        setAuthor(source.getAuthor());
        if (source.question_ != null) {
            setQuestion((Question)source.getQuestion().clone());
        }
        if (source.constraint_ != null) {
            setConstraint((Constraint)source.getConstraint().clone());
        }
        setAnswer(source.getAnswer());
        setHint(source.getHint());
        setHidden(source.getHidden());
        setSeed(source.getSeed());
        setComment(source.getComment());
    }

    /**
     * Initializes the <code>Problem</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Problem</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Problem</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        size_ = URelaxer.getAttributePropertyAsString(element, "size");
        name_ = URelaxer.getAttributePropertyAsString(element, "name");
        author_ = URelaxer.getAttributePropertyAsString(element, "author");
        if (Question.isMatch(stack)) {
            setQuestion(new Question(stack));
        }
        setConstraint(new Constraint(stack));
        answer_ = URelaxer.getElementPropertyAsStringByStack(stack, "answer");
        hint_ = URelaxer.getElementPropertyAsStringByStack(stack, "hint");
        hidden_ = URelaxer.getElementPropertyAsStringByStack(stack, "hidden");
        seed_ = URelaxer.getElementPropertyAsStringByStack(stack, "seed");
        comment_ = URelaxer.getElementPropertyAsStringByStack(stack, "comment");
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Problem(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("problem");
        int size;
        if (this.size_ != null) {
            URelaxer.setAttributePropertyByString(element, "size", this.size_);
        }
        if (this.name_ != null) {
            URelaxer.setAttributePropertyByString(element, "name", this.name_);
        }
        if (this.author_ != null) {
            URelaxer.setAttributePropertyByString(element, "author", this.author_);
        }
        if (this.question_ != null) {
            this.question_.makeElement(element);
        }
        this.constraint_.makeElement(element);
        if (this.answer_ != null) {
            URelaxer.setElementPropertyByString(element, "answer", this.answer_);
        }
        if (this.hint_ != null) {
            URelaxer.setElementPropertyByString(element, "hint", this.hint_);
        }
        if (this.hidden_ != null) {
            URelaxer.setElementPropertyByString(element, "hidden", this.hidden_);
        }
        if (this.seed_ != null) {
            URelaxer.setElementPropertyByString(element, "seed", this.seed_);
        }
        if (this.comment_ != null) {
            URelaxer.setElementPropertyByString(element, "comment", this.comment_);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Problem</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Problem</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>size</b>.
     *
     * @return String
     */
    public final String getSize() {
        return (size_);
    }

    /**
     * Sets the String property <b>size</b>.
     *
     * @param sizeValue
     */
    public final void setSize(String sizeValue) {
        this.size_ = sizeValue;
    }

    /**
     * Gets the String property <b>name</b>.
     *
     * @return String
     */
    public final String getName() {
        return (name_);
    }

    /**
     * Sets the String property <b>name</b>.
     *
     * @param name
     */
    public final void setName(String name) {
        this.name_ = name;
    }

    /**
     * Gets the String property <b>author</b>.
     *
     * @return String
     */
    public final String getAuthor() {
        return (author_);
    }

    /**
     * Sets the String property <b>author</b>.
     *
     * @param author
     */
    public final void setAuthor(String author) {
        this.author_ = author;
    }

    /**
     * Gets the Question property <b>Question</b>.
     *
     * @return Question
     */
    public final Question getQuestion() {
        return (question_);
    }

    /**
     * Sets the Question property <b>Question</b>.
     *
     * @param question
     */
    public final void setQuestion(Question question) {
        this.question_ = question;
    }

    /**
     * Gets the Constraint property <b>Constraint</b>.
     *
     * @return Constraint
     */
    public final Constraint getConstraint() {
        return (constraint_);
    }

    /**
     * Sets the Constraint property <b>Constraint</b>.
     *
     * @param constraint
     */
    public final void setConstraint(Constraint constraint) {
        this.constraint_ = constraint;
    }

    /**
     * Gets the String property <b>answer</b>.
     *
     * @return String
     */
    public final String getAnswer() {
        return (answer_);
    }

    /**
     * Sets the String property <b>answer</b>.
     *
     * @param answer
     */
    public final void setAnswer(String answer) {
        this.answer_ = answer;
    }

    /**
     * Gets the String property <b>hint</b>.
     *
     * @return String
     */
    public final String getHint() {
        return (hint_);
    }

    /**
     * Sets the String property <b>hint</b>.
     *
     * @param hint
     */
    public final void setHint(String hint) {
        this.hint_ = hint;
    }

    /**
     * Gets the String property <b>hidden</b>.
     *
     * @return String
     */
    public final String getHidden() {
        return (hidden_);
    }

    /**
     * Sets the String property <b>hidden</b>.
     *
     * @param hidden
     */
    public final void setHidden(String hidden) {
        this.hidden_ = hidden;
    }

    /**
     * Gets the String property <b>seed</b>.
     *
     * @return String
     */
    public final String getSeed() {
        return (seed_);
    }

    /**
     * Sets the String property <b>seed</b>.
     *
     * @param seed
     */
    public final void setSeed(String seed) {
        this.seed_ = seed;
    }

    /**
     * Gets the String property <b>comment</b>.
     *
     * @return String
     */
    public final String getComment() {
        return (comment_);
    }

    /**
     * Sets the String property <b>comment</b>.
     *
     * @param comment
     */
    public final void setComment(String comment) {
        this.comment_ = comment;
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<problem");
        if (size_ != null) {
            buffer.append(" size=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getSize())));
            buffer.append("\"");
        }
        if (name_ != null) {
            buffer.append(" name=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.append("\"");
        }
        if (author_ != null) {
            buffer.append(" author=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getAuthor())));
            buffer.append("\"");
        }
        buffer.append(">");
        if (question_ != null) {
            question_.makeTextElement(buffer);
        }
        constraint_.makeTextElement(buffer);
        if (answer_ != null) {
            buffer.append("<answer>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getAnswer())));
            buffer.append("</answer>");
        }
        if (hint_ != null) {
            buffer.append("<hint>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getHint())));
            buffer.append("</hint>");
        }
        if (hidden_ != null) {
            buffer.append("<hidden>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getHidden())));
            buffer.append("</hidden>");
        }
        if (seed_ != null) {
            buffer.append("<seed>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getSeed())));
            buffer.append("</seed>");
        }
        if (comment_ != null) {
            buffer.append("<comment>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getComment())));
            buffer.append("</comment>");
        }
        buffer.append("</problem>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<problem");
        if (size_ != null) {
            buffer.write(" size=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getSize())));
            buffer.write("\"");
        }
        if (name_ != null) {
            buffer.write(" name=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.write("\"");
        }
        if (author_ != null) {
            buffer.write(" author=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getAuthor())));
            buffer.write("\"");
        }
        buffer.write(">");
        if (question_ != null) {
            question_.makeTextElement(buffer);
        }
        constraint_.makeTextElement(buffer);
        if (answer_ != null) {
            buffer.write("<answer>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getAnswer())));
            buffer.write("</answer>");
        }
        if (hint_ != null) {
            buffer.write("<hint>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getHint())));
            buffer.write("</hint>");
        }
        if (hidden_ != null) {
            buffer.write("<hidden>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getHidden())));
            buffer.write("</hidden>");
        }
        if (seed_ != null) {
            buffer.write("<seed>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getSeed())));
            buffer.write("</seed>");
        }
        if (comment_ != null) {
            buffer.write("<comment>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getComment())));
            buffer.write("</comment>");
        }
        buffer.write("</problem>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<problem");
        if (size_ != null) {
            buffer.print(" size=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getSize())));
            buffer.print("\"");
        }
        if (name_ != null) {
            buffer.print(" name=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.print("\"");
        }
        if (author_ != null) {
            buffer.print(" author=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getAuthor())));
            buffer.print("\"");
        }
        buffer.print(">");
        if (question_ != null) {
            question_.makeTextElement(buffer);
        }
        constraint_.makeTextElement(buffer);
        if (answer_ != null) {
            buffer.print("<answer>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getAnswer())));
            buffer.print("</answer>");
        }
        if (hint_ != null) {
            buffer.print("<hint>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getHint())));
            buffer.print("</hint>");
        }
        if (hidden_ != null) {
            buffer.print("<hidden>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getHidden())));
            buffer.print("</hidden>");
        }
        if (seed_ != null) {
            buffer.print("<seed>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getSeed())));
            buffer.print("</seed>");
        }
        if (comment_ != null) {
            buffer.print("<comment>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getComment())));
            buffer.print("</comment>");
        }
        buffer.print("</problem>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSizeAsString() {
        return (URelaxer.getString(getSize()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getNameAsString() {
        return (URelaxer.getString(getName()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAuthorAsString() {
        return (URelaxer.getString(getAuthor()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAnswerAsString() {
        return (URelaxer.getString(getAnswer()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHintAsString() {
        return (URelaxer.getString(getHint()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHiddenAsString() {
        return (URelaxer.getString(getHidden()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getSeedAsString() {
        return (URelaxer.getString(getSeed()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCommentAsString() {
        return (URelaxer.getString(getComment()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSizeByString(String string) {
        setSize(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setNameByString(String string) {
        setName(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAuthorByString(String string) {
        setAuthor(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAnswerByString(String string) {
        setAnswer(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHintByString(String string) {
        setHint(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHiddenByString(String string) {
        setHidden(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setSeedByString(String string) {
        setSeed(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCommentByString(String string) {
        setComment(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Problem</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "problem")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!URelaxer.hasAttributeHungry(target, "size")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "name")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "author")) {
            return (false);
        }
        $match$ = true;
        if (Question.isMatchHungry(target)) {
        }
        if (!Constraint.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "answer")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "hint")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "hidden")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "seed")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "comment")) {
                target.popElement();
            }
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Problem</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Problem</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
